/* arraylist.c generated by valac 0.25.3.11-62b1e, the Vala compiler
 * generated from arraylist.vala, do not modify */

/* arraylist.vala
 *
 * Copyright (C) 2004-2005  Novell, Inc
 * Copyright (C) 2005  David Waite
 * Copyright (C) 2007-2008  Jürg Billeter
 * Copyright (C) 2009  Didier Villevalois
 * Copyright (C) 2010-2014  Maciej Piechotka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 * 	Didier 'Ptitjes Villevalois <ptitjes@free.fr>
 */

#include <glib.h>
#include <glib-object.h>
#include <string.h>
#include <float.h>
#include <math.h>


#define GEE_TYPE_TRAVERSABLE (gee_traversable_get_type ())
#define GEE_TRAVERSABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_TRAVERSABLE, GeeTraversable))
#define GEE_IS_TRAVERSABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_TRAVERSABLE))
#define GEE_TRAVERSABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_TRAVERSABLE, GeeTraversableIface))

typedef struct _GeeTraversable GeeTraversable;
typedef struct _GeeTraversableIface GeeTraversableIface;

#define GEE_TRAVERSABLE_TYPE_STREAM (gee_traversable_stream_get_type ())

#define GEE_TYPE_LAZY (gee_lazy_get_type ())
#define GEE_LAZY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_LAZY, GeeLazy))
#define GEE_LAZY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_LAZY, GeeLazyClass))
#define GEE_IS_LAZY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_LAZY))
#define GEE_IS_LAZY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_LAZY))
#define GEE_LAZY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_LAZY, GeeLazyClass))

typedef struct _GeeLazy GeeLazy;
typedef struct _GeeLazyClass GeeLazyClass;

#define GEE_TYPE_ITERATOR (gee_iterator_get_type ())
#define GEE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERATOR, GeeIterator))
#define GEE_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERATOR))
#define GEE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERATOR, GeeIteratorIface))

typedef struct _GeeIterator GeeIterator;
typedef struct _GeeIteratorIface GeeIteratorIface;

#define GEE_TYPE_ITERABLE (gee_iterable_get_type ())
#define GEE_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERABLE, GeeIterable))
#define GEE_IS_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERABLE))
#define GEE_ITERABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERABLE, GeeIterableIface))

typedef struct _GeeIterable GeeIterable;
typedef struct _GeeIterableIface GeeIterableIface;

#define GEE_TYPE_COLLECTION (gee_collection_get_type ())
#define GEE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION, GeeCollection))
#define GEE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION))
#define GEE_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_COLLECTION, GeeCollectionIface))

typedef struct _GeeCollection GeeCollection;
typedef struct _GeeCollectionIface GeeCollectionIface;

#define GEE_TYPE_ABSTRACT_COLLECTION (gee_abstract_collection_get_type ())
#define GEE_ABSTRACT_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollection))
#define GEE_ABSTRACT_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollectionClass))
#define GEE_IS_ABSTRACT_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ABSTRACT_COLLECTION))
#define GEE_IS_ABSTRACT_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_ABSTRACT_COLLECTION))
#define GEE_ABSTRACT_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollectionClass))

typedef struct _GeeAbstractCollection GeeAbstractCollection;
typedef struct _GeeAbstractCollectionClass GeeAbstractCollectionClass;
typedef struct _GeeAbstractCollectionPrivate GeeAbstractCollectionPrivate;

#define GEE_TYPE_LIST (gee_list_get_type ())
#define GEE_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_LIST, GeeList))
#define GEE_IS_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_LIST))
#define GEE_LIST_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_LIST, GeeListIface))

typedef struct _GeeList GeeList;
typedef struct _GeeListIface GeeListIface;

#define GEE_TYPE_LIST_ITERATOR (gee_list_iterator_get_type ())
#define GEE_LIST_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_LIST_ITERATOR, GeeListIterator))
#define GEE_IS_LIST_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_LIST_ITERATOR))
#define GEE_LIST_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_LIST_ITERATOR, GeeListIteratorIface))

typedef struct _GeeListIterator GeeListIterator;
typedef struct _GeeListIteratorIface GeeListIteratorIface;

#define GEE_TYPE_ABSTRACT_LIST (gee_abstract_list_get_type ())
#define GEE_ABSTRACT_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ABSTRACT_LIST, GeeAbstractList))
#define GEE_ABSTRACT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_ABSTRACT_LIST, GeeAbstractListClass))
#define GEE_IS_ABSTRACT_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ABSTRACT_LIST))
#define GEE_IS_ABSTRACT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_ABSTRACT_LIST))
#define GEE_ABSTRACT_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_ABSTRACT_LIST, GeeAbstractListClass))

typedef struct _GeeAbstractList GeeAbstractList;
typedef struct _GeeAbstractListClass GeeAbstractListClass;
typedef struct _GeeAbstractListPrivate GeeAbstractListPrivate;

#define GEE_TYPE_BIDIR_LIST (gee_bidir_list_get_type ())
#define GEE_BIDIR_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_BIDIR_LIST, GeeBidirList))
#define GEE_IS_BIDIR_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_BIDIR_LIST))
#define GEE_BIDIR_LIST_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_BIDIR_LIST, GeeBidirListIface))

typedef struct _GeeBidirList GeeBidirList;
typedef struct _GeeBidirListIface GeeBidirListIface;

#define GEE_TYPE_BIDIR_ITERATOR (gee_bidir_iterator_get_type ())
#define GEE_BIDIR_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_BIDIR_ITERATOR, GeeBidirIterator))
#define GEE_IS_BIDIR_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_BIDIR_ITERATOR))
#define GEE_BIDIR_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_BIDIR_ITERATOR, GeeBidirIteratorIface))

typedef struct _GeeBidirIterator GeeBidirIterator;
typedef struct _GeeBidirIteratorIface GeeBidirIteratorIface;

#define GEE_TYPE_BIDIR_LIST_ITERATOR (gee_bidir_list_iterator_get_type ())
#define GEE_BIDIR_LIST_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_BIDIR_LIST_ITERATOR, GeeBidirListIterator))
#define GEE_IS_BIDIR_LIST_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_BIDIR_LIST_ITERATOR))
#define GEE_BIDIR_LIST_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_BIDIR_LIST_ITERATOR, GeeBidirListIteratorIface))

typedef struct _GeeBidirListIterator GeeBidirListIterator;
typedef struct _GeeBidirListIteratorIface GeeBidirListIteratorIface;

#define GEE_TYPE_ABSTRACT_BIDIR_LIST (gee_abstract_bidir_list_get_type ())
#define GEE_ABSTRACT_BIDIR_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ABSTRACT_BIDIR_LIST, GeeAbstractBidirList))
#define GEE_ABSTRACT_BIDIR_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_ABSTRACT_BIDIR_LIST, GeeAbstractBidirListClass))
#define GEE_IS_ABSTRACT_BIDIR_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ABSTRACT_BIDIR_LIST))
#define GEE_IS_ABSTRACT_BIDIR_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_ABSTRACT_BIDIR_LIST))
#define GEE_ABSTRACT_BIDIR_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_ABSTRACT_BIDIR_LIST, GeeAbstractBidirListClass))

typedef struct _GeeAbstractBidirList GeeAbstractBidirList;
typedef struct _GeeAbstractBidirListClass GeeAbstractBidirListClass;
typedef struct _GeeAbstractBidirListPrivate GeeAbstractBidirListPrivate;

#define GEE_TYPE_ARRAY_LIST (gee_array_list_get_type ())
#define GEE_ARRAY_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ARRAY_LIST, GeeArrayList))
#define GEE_ARRAY_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_ARRAY_LIST, GeeArrayListClass))
#define GEE_IS_ARRAY_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ARRAY_LIST))
#define GEE_IS_ARRAY_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_ARRAY_LIST))
#define GEE_ARRAY_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_ARRAY_LIST, GeeArrayListClass))

typedef struct _GeeArrayList GeeArrayList;
typedef struct _GeeArrayListClass GeeArrayListClass;
typedef struct _GeeArrayListPrivate GeeArrayListPrivate;

#define GEE_FUNCTIONS_TYPE_EQUAL_DATA_FUNC_CLOSURE (gee_functions_equal_data_func_closure_get_type ())
#define GEE_FUNCTIONS_EQUAL_DATA_FUNC_CLOSURE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_FUNCTIONS_TYPE_EQUAL_DATA_FUNC_CLOSURE, GeeFunctionsEqualDataFuncClosure))
#define GEE_FUNCTIONS_EQUAL_DATA_FUNC_CLOSURE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_FUNCTIONS_TYPE_EQUAL_DATA_FUNC_CLOSURE, GeeFunctionsEqualDataFuncClosureClass))
#define GEE_FUNCTIONS_IS_EQUAL_DATA_FUNC_CLOSURE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_FUNCTIONS_TYPE_EQUAL_DATA_FUNC_CLOSURE))
#define GEE_FUNCTIONS_IS_EQUAL_DATA_FUNC_CLOSURE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_FUNCTIONS_TYPE_EQUAL_DATA_FUNC_CLOSURE))
#define GEE_FUNCTIONS_EQUAL_DATA_FUNC_CLOSURE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_FUNCTIONS_TYPE_EQUAL_DATA_FUNC_CLOSURE, GeeFunctionsEqualDataFuncClosureClass))

typedef struct _GeeFunctionsEqualDataFuncClosure GeeFunctionsEqualDataFuncClosure;
typedef struct _GeeFunctionsEqualDataFuncClosureClass GeeFunctionsEqualDataFuncClosureClass;
#define _gee_functions_equal_data_func_closure_unref0(var) ((var == NULL) ? NULL : (var = (gee_functions_equal_data_func_closure_unref (var), NULL)))

#define GEE_ARRAY_LIST_TYPE_ITERATOR (gee_array_list_iterator_get_type ())
#define GEE_ARRAY_LIST_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_ARRAY_LIST_TYPE_ITERATOR, GeeArrayListIterator))
#define GEE_ARRAY_LIST_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_ARRAY_LIST_TYPE_ITERATOR, GeeArrayListIteratorClass))
#define GEE_ARRAY_LIST_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_ARRAY_LIST_TYPE_ITERATOR))
#define GEE_ARRAY_LIST_IS_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_ARRAY_LIST_TYPE_ITERATOR))
#define GEE_ARRAY_LIST_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_ARRAY_LIST_TYPE_ITERATOR, GeeArrayListIteratorClass))

typedef struct _GeeArrayListIterator GeeArrayListIterator;
typedef struct _GeeArrayListIteratorClass GeeArrayListIteratorClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_destroy_func0(var) (((var == NULL) || (g_destroy_func == NULL)) ? NULL : (var = (g_destroy_func (var), NULL)))
typedef struct _GeeFunctionsEqualDataFuncClosurePrivate GeeFunctionsEqualDataFuncClosurePrivate;
typedef struct _GeeArrayListIteratorPrivate GeeArrayListIteratorPrivate;
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

typedef gboolean (*GeeForallFunc) (gpointer g, void* user_data);
typedef enum  {
	GEE_TRAVERSABLE_STREAM_YIELD,
	GEE_TRAVERSABLE_STREAM_CONTINUE,
	GEE_TRAVERSABLE_STREAM_END,
	GEE_TRAVERSABLE_STREAM_WAIT
} GeeTraversableStream;

typedef GeeTraversableStream (*GeeStreamFunc) (GeeTraversableStream state, GeeLazy* g, GeeLazy** lazy, void* user_data);
struct _GeeIteratorIface {
	GTypeInterface parent_iface;
	gboolean (*next) (GeeIterator* self);
	gboolean (*has_next) (GeeIterator* self);
	gpointer (*get) (GeeIterator* self);
	void (*remove) (GeeIterator* self);
	gboolean (*get_valid) (GeeIterator* self);
	gboolean (*get_read_only) (GeeIterator* self);
};

typedef gpointer (*GeeFoldFunc) (gpointer g, gpointer a, void* user_data);
typedef gpointer (*GeeMapFunc) (gpointer g, void* user_data);
typedef gboolean (*GeePredicate) (gconstpointer g, void* user_data);
typedef GeeIterator* (*GeeFlatMapFunc) (gpointer g, void* user_data);
struct _GeeTraversableIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeTraversable* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeTraversable* self);
	GDestroyNotify (*get_g_destroy_func) (GeeTraversable* self);
	gboolean (*foreach) (GeeTraversable* self, GeeForallFunc f, void* f_target);
	GeeIterator* (*stream) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeStreamFunc f, void* f_target, GDestroyNotify f_target_destroy_notify);
	gpointer (*fold) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFoldFunc f, void* f_target, gpointer seed);
	GeeIterator* (*map) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeMapFunc f, void* f_target);
	GeeIterator* (*scan) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFoldFunc f, void* f_target, gpointer seed);
	GeeIterator* (*filter) (GeeTraversable* self, GeePredicate pred, void* pred_target, GDestroyNotify pred_target_destroy_notify);
	GeeIterator* (*chop) (GeeTraversable* self, gint offset, gint length);
	GType (*get_element_type) (GeeTraversable* self);
	GeeIterator* (*flat_map) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFlatMapFunc f, void* f_target, GDestroyNotify f_target_destroy_notify);
	GeeIterator** (*tee) (GeeTraversable* self, guint forks, int* result_length1);
};

struct _GeeIterableIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeIterable* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeIterable* self);
	GDestroyNotify (*get_g_destroy_func) (GeeIterable* self);
	GeeIterator* (*iterator) (GeeIterable* self);
};

struct _GeeCollectionIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeCollection* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeCollection* self);
	GDestroyNotify (*get_g_destroy_func) (GeeCollection* self);
	gboolean (*contains) (GeeCollection* self, gconstpointer item);
	gboolean (*add) (GeeCollection* self, gconstpointer item);
	gboolean (*remove) (GeeCollection* self, gconstpointer item);
	void (*clear) (GeeCollection* self);
	gboolean (*add_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*contains_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*remove_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*retain_all) (GeeCollection* self, GeeCollection* collection);
	gpointer* (*to_array) (GeeCollection* self, int* result_length1);
	gint (*get_size) (GeeCollection* self);
	gboolean (*get_is_empty) (GeeCollection* self);
	gboolean (*get_read_only) (GeeCollection* self);
	GeeCollection* (*get_read_only_view) (GeeCollection* self);
	gboolean (*add_all_array) (GeeCollection* self, gpointer* array, int array_length1);
	gboolean (*contains_all_array) (GeeCollection* self, gpointer* array, int array_length1);
	gboolean (*remove_all_array) (GeeCollection* self, gpointer* array, int array_length1);
	gboolean (*add_all_iterator) (GeeCollection* self, GeeIterator* iter);
	gboolean (*contains_all_iterator) (GeeCollection* self, GeeIterator* iter);
	gboolean (*remove_all_iterator) (GeeCollection* self, GeeIterator* iter);
};

struct _GeeAbstractCollection {
	GObject parent_instance;
	GeeAbstractCollectionPrivate * priv;
};

struct _GeeAbstractCollectionClass {
	GObjectClass parent_class;
	gboolean (*contains) (GeeAbstractCollection* self, gconstpointer item);
	gboolean (*add) (GeeAbstractCollection* self, gconstpointer item);
	gboolean (*remove) (GeeAbstractCollection* self, gconstpointer item);
	void (*clear) (GeeAbstractCollection* self);
	GeeIterator* (*iterator) (GeeAbstractCollection* self);
	gboolean (*foreach) (GeeAbstractCollection* self, GeeForallFunc f, void* f_target);
	void (*reserved0) (GeeAbstractCollection* self);
	void (*reserved1) (GeeAbstractCollection* self);
	void (*reserved2) (GeeAbstractCollection* self);
	void (*reserved3) (GeeAbstractCollection* self);
	void (*reserved4) (GeeAbstractCollection* self);
	void (*reserved5) (GeeAbstractCollection* self);
	void (*reserved6) (GeeAbstractCollection* self);
	void (*reserved7) (GeeAbstractCollection* self);
	void (*reserved8) (GeeAbstractCollection* self);
	void (*reserved9) (GeeAbstractCollection* self);
	gint (*get_size) (GeeAbstractCollection* self);
	gboolean (*get_read_only) (GeeAbstractCollection* self);
	GeeCollection* (*get_read_only_view) (GeeAbstractCollection* self);
};

struct _GeeListIteratorIface {
	GTypeInterface parent_iface;
	void (*set) (GeeListIterator* self, gconstpointer item);
	void (*add) (GeeListIterator* self, gconstpointer item);
	gint (*index) (GeeListIterator* self);
};

struct _GeeListIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeList* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeList* self);
	GDestroyNotify (*get_g_destroy_func) (GeeList* self);
	GeeListIterator* (*list_iterator) (GeeList* self);
	gpointer (*get) (GeeList* self, gint index);
	void (*set) (GeeList* self, gint index, gconstpointer item);
	gint (*index_of) (GeeList* self, gconstpointer item);
	void (*insert) (GeeList* self, gint index, gconstpointer item);
	gpointer (*remove_at) (GeeList* self, gint index);
	GeeList* (*slice) (GeeList* self, gint start, gint stop);
	gpointer (*first) (GeeList* self);
	gpointer (*last) (GeeList* self);
	void (*insert_all) (GeeList* self, gint index, GeeCollection* collection);
	void (*sort) (GeeList* self, GCompareDataFunc compare_func, void* compare_func_target, GDestroyNotify compare_func_target_destroy_notify);
	GeeList* (*get_read_only_view) (GeeList* self);
};

struct _GeeAbstractList {
	GeeAbstractCollection parent_instance;
	GeeAbstractListPrivate * priv;
};

struct _GeeAbstractListClass {
	GeeAbstractCollectionClass parent_class;
	GeeListIterator* (*list_iterator) (GeeAbstractList* self);
	gpointer (*get) (GeeAbstractList* self, gint index);
	void (*set) (GeeAbstractList* self, gint index, gconstpointer item);
	gint (*index_of) (GeeAbstractList* self, gconstpointer item);
	void (*insert) (GeeAbstractList* self, gint index, gconstpointer item);
	gpointer (*remove_at) (GeeAbstractList* self, gint index);
	GeeList* (*slice) (GeeAbstractList* self, gint start, gint stop);
	void (*reserved0) (GeeAbstractList* self);
	void (*reserved1) (GeeAbstractList* self);
	void (*reserved2) (GeeAbstractList* self);
	void (*reserved3) (GeeAbstractList* self);
	void (*reserved4) (GeeAbstractList* self);
	void (*reserved5) (GeeAbstractList* self);
	void (*reserved6) (GeeAbstractList* self);
	void (*reserved7) (GeeAbstractList* self);
	void (*reserved8) (GeeAbstractList* self);
	void (*reserved9) (GeeAbstractList* self);
	GeeList* (*get_read_only_view) (GeeAbstractList* self);
};

struct _GeeBidirIteratorIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeBidirIterator* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeBidirIterator* self);
	GDestroyNotify (*get_g_destroy_func) (GeeBidirIterator* self);
	gboolean (*previous) (GeeBidirIterator* self);
	gboolean (*has_previous) (GeeBidirIterator* self);
	gboolean (*first) (GeeBidirIterator* self);
	gboolean (*last) (GeeBidirIterator* self);
};

struct _GeeBidirListIteratorIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeBidirListIterator* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeBidirListIterator* self);
	GDestroyNotify (*get_g_destroy_func) (GeeBidirListIterator* self);
	void (*insert) (GeeBidirListIterator* self, gconstpointer item);
};

struct _GeeBidirListIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeBidirList* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeBidirList* self);
	GDestroyNotify (*get_g_destroy_func) (GeeBidirList* self);
	GeeBidirListIterator* (*bidir_list_iterator) (GeeBidirList* self);
	GeeBidirList* (*get_read_only_view) (GeeBidirList* self);
};

struct _GeeAbstractBidirList {
	GeeAbstractList parent_instance;
	GeeAbstractBidirListPrivate * priv;
};

struct _GeeAbstractBidirListClass {
	GeeAbstractListClass parent_class;
	GeeBidirListIterator* (*bidir_list_iterator) (GeeAbstractBidirList* self);
	void (*reserved0) (GeeAbstractBidirList* self);
	void (*reserved1) (GeeAbstractBidirList* self);
	void (*reserved2) (GeeAbstractBidirList* self);
	void (*reserved3) (GeeAbstractBidirList* self);
	void (*reserved4) (GeeAbstractBidirList* self);
	void (*reserved5) (GeeAbstractBidirList* self);
	void (*reserved6) (GeeAbstractBidirList* self);
	void (*reserved7) (GeeAbstractBidirList* self);
	void (*reserved8) (GeeAbstractBidirList* self);
	void (*reserved9) (GeeAbstractBidirList* self);
	GeeBidirList* (*get_read_only_view) (GeeAbstractBidirList* self);
};

struct _GeeArrayList {
	GeeAbstractBidirList parent_instance;
	GeeArrayListPrivate * priv;
	gpointer* _items;
	gint _items_length1;
	gint __items_size_;
	gint _size;
};

struct _GeeArrayListClass {
	GeeAbstractBidirListClass parent_class;
};

struct _GeeArrayListPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
	GeeFunctionsEqualDataFuncClosure* _equal_func;
	gint _stamp;
};

typedef gboolean (*GeeEqualDataFunc) (gconstpointer a, gconstpointer b, void* user_data);
struct _GeeFunctionsEqualDataFuncClosure {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GeeFunctionsEqualDataFuncClosurePrivate * priv;
	GeeEqualDataFunc func;
	gpointer func_target;
	GDestroyNotify func_target_destroy_notify;
};

struct _GeeFunctionsEqualDataFuncClosureClass {
	GTypeClass parent_class;
	void (*finalize) (GeeFunctionsEqualDataFuncClosure *self);
};

struct _GeeArrayListIterator {
	GObject parent_instance;
	GeeArrayListIteratorPrivate * priv;
	GeeArrayList* _list;
	gint _index;
	gboolean _removed;
	gint _stamp;
};

struct _GeeArrayListIteratorClass {
	GObjectClass parent_class;
};

struct _GeeArrayListIteratorPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
};


static gpointer gee_array_list_parent_class = NULL;
static gpointer gee_array_list_iterator_parent_class = NULL;
static GeeTraversableIface* gee_array_list_iterator_gee_traversable_parent_iface = NULL;
static GeeIteratorIface* gee_array_list_iterator_gee_iterator_parent_iface = NULL;
static GeeBidirIteratorIface* gee_array_list_iterator_gee_bidir_iterator_parent_iface = NULL;
static GeeListIteratorIface* gee_array_list_iterator_gee_list_iterator_parent_iface = NULL;
static GeeBidirListIteratorIface* gee_array_list_iterator_gee_bidir_list_iterator_parent_iface = NULL;

GType gee_traversable_stream_get_type (void) G_GNUC_CONST;
gpointer gee_lazy_ref (gpointer instance);
void gee_lazy_unref (gpointer instance);
GParamSpec* gee_param_spec_lazy (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void gee_value_set_lazy (GValue* value, gpointer v_object);
void gee_value_take_lazy (GValue* value, gpointer v_object);
gpointer gee_value_get_lazy (const GValue* value);
GType gee_lazy_get_type (void) G_GNUC_CONST;
GType gee_iterator_get_type (void) G_GNUC_CONST;
GType gee_traversable_get_type (void) G_GNUC_CONST;
GType gee_iterable_get_type (void) G_GNUC_CONST;
GType gee_collection_get_type (void) G_GNUC_CONST;
GType gee_abstract_collection_get_type (void) G_GNUC_CONST;
GType gee_list_iterator_get_type (void) G_GNUC_CONST;
GType gee_list_get_type (void) G_GNUC_CONST;
GType gee_abstract_list_get_type (void) G_GNUC_CONST;
GType gee_bidir_iterator_get_type (void) G_GNUC_CONST;
GType gee_bidir_list_iterator_get_type (void) G_GNUC_CONST;
GType gee_bidir_list_get_type (void) G_GNUC_CONST;
GType gee_abstract_bidir_list_get_type (void) G_GNUC_CONST;
GType gee_array_list_get_type (void) G_GNUC_CONST;
G_GNUC_INTERNAL gpointer gee_functions_equal_data_func_closure_ref (gpointer instance);
G_GNUC_INTERNAL void gee_functions_equal_data_func_closure_unref (gpointer instance);
G_GNUC_INTERNAL GParamSpec* gee_functions_param_spec_equal_data_func_closure (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
G_GNUC_INTERNAL void gee_functions_value_set_equal_data_func_closure (GValue* value, gpointer v_object) G_GNUC_UNUSED;
G_GNUC_INTERNAL void gee_functions_value_take_equal_data_func_closure (GValue* value, gpointer v_object);
G_GNUC_INTERNAL gpointer gee_functions_value_get_equal_data_func_closure (const GValue* value) G_GNUC_UNUSED;
G_GNUC_INTERNAL GType gee_functions_equal_data_func_closure_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
#define GEE_ARRAY_LIST_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_TYPE_ARRAY_LIST, GeeArrayListPrivate))
enum  {
	GEE_ARRAY_LIST_DUMMY_PROPERTY,
	GEE_ARRAY_LIST_G_TYPE,
	GEE_ARRAY_LIST_G_DUP_FUNC,
	GEE_ARRAY_LIST_G_DESTROY_FUNC,
	GEE_ARRAY_LIST_SIZE,
	GEE_ARRAY_LIST_READ_ONLY
};
GeeArrayList* gee_array_list_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeEqualDataFunc equal_func, void* equal_func_target, GDestroyNotify equal_func_target_destroy_notify);
GeeArrayList* gee_array_list_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeEqualDataFunc equal_func, void* equal_func_target, GDestroyNotify equal_func_target_destroy_notify);
GeeAbstractBidirList* gee_abstract_bidir_list_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
GeeEqualDataFunc gee_functions_get_equal_func_for (GType t, void** result_target, GDestroyNotify* result_target_destroy_notify);
G_GNUC_INTERNAL GeeFunctionsEqualDataFuncClosure* gee_functions_equal_data_func_closure_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeEqualDataFunc func, void* func_target, GDestroyNotify func_target_destroy_notify);
G_GNUC_INTERNAL GeeFunctionsEqualDataFuncClosure* gee_functions_equal_data_func_closure_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeEqualDataFunc func, void* func_target, GDestroyNotify func_target_destroy_notify);
GeeArrayList* gee_array_list_new_wrap (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gpointer* items, int items_length1, GeeEqualDataFunc equal_func, void* equal_func_target, GDestroyNotify equal_func_target_destroy_notify);
GeeArrayList* gee_array_list_construct_wrap (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gpointer* items, int items_length1, GeeEqualDataFunc equal_func, void* equal_func_target, GDestroyNotify equal_func_target_destroy_notify);
static gpointer* gee_array_list_do_wrap (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gpointer* data, int data_length1, int* result_length1);
G_GNUC_INTERNAL GeeArrayList* gee_array_list_new_with_closure (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeFunctionsEqualDataFuncClosure* equal_func);
G_GNUC_INTERNAL GeeArrayList* gee_array_list_construct_with_closure (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeFunctionsEqualDataFuncClosure* equal_func);
static gboolean gee_array_list_real_foreach (GeeAbstractCollection* base, GeeForallFunc f, void* f_target);
static GeeIterator* gee_array_list_real_iterator (GeeAbstractCollection* base);
static GeeArrayListIterator* gee_array_list_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeArrayList* list);
static GeeArrayListIterator* gee_array_list_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeArrayList* list);
static GType gee_array_list_iterator_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static GeeListIterator* gee_array_list_real_list_iterator (GeeAbstractList* base);
static GeeBidirListIterator* gee_array_list_real_bidir_list_iterator (GeeAbstractBidirList* base);
static gboolean gee_array_list_real_contains (GeeAbstractCollection* base, gconstpointer item);
gint gee_abstract_list_index_of (GeeAbstractList* self, gconstpointer item);
static gint gee_array_list_real_index_of (GeeAbstractList* base, gconstpointer item);
GeeEqualDataFunc gee_array_list_get_equal_func (GeeArrayList* self, gpointer* result_target);
static gpointer gee_array_list_real_get (GeeAbstractList* base, gint index);
static void gee_array_list_real_set (GeeAbstractList* base, gint index, gconstpointer item);
static gboolean gee_array_list_real_add (GeeAbstractCollection* base, gconstpointer item);
static void gee_array_list_grow_if_needed (GeeArrayList* self, gint new_count);
static void gee_array_list_real_insert (GeeAbstractList* base, gint index, gconstpointer item);
static void gee_array_list_shift (GeeArrayList* self, gint start, gint delta);
static gboolean gee_array_list_real_remove (GeeAbstractCollection* base, gconstpointer item);
gpointer gee_abstract_list_remove_at (GeeAbstractList* self, gint index);
static gpointer gee_array_list_real_remove_at (GeeAbstractList* base, gint index);
static void gee_array_list_real_clear (GeeAbstractCollection* base);
static GeeList* gee_array_list_real_slice (GeeAbstractList* base, gint start, gint stop);
gboolean gee_abstract_collection_add (GeeAbstractCollection* self, gconstpointer item);
gpointer gee_abstract_list_get (GeeAbstractList* self, gint index);
gboolean gee_array_list_add_all (GeeArrayList* self, GeeCollection* collection);
gboolean gee_collection_get_is_empty (GeeCollection* self);
gint gee_collection_get_size (GeeCollection* self);
gboolean gee_traversable_foreach (GeeTraversable* self, GeeForallFunc f, void* f_target);
static gboolean __lambda43_ (GeeArrayList* self, gpointer item);
static gboolean ___lambda43__gee_forall_func (gpointer g, gpointer self);
static void gee_array_list_set_capacity (GeeArrayList* self, gint value);
static gpointer* gee_array_list_wrap_bool (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gboolean* data, int data_length1, int* result_length1);
static gpointer* gee_array_list_wrap_char (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gchar* data, int data_length1, int* result_length1);
static gpointer* gee_array_list_wrap_uchar (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guchar* data, int data_length1, int* result_length1);
static gpointer* gee_array_list_wrap_int (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gint* data, int data_length1, int* result_length1);
static gpointer* gee_array_list_wrap_uint (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guint* data, int data_length1, int* result_length1);
static gpointer* gee_array_list_wrap_int64 (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gint64* data, int data_length1, int* result_length1);
static gpointer* gee_array_list_wrap_uint64 (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guint64* data, int data_length1, int* result_length1);
static gpointer* gee_array_list_wrap_long (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, glong* data, int data_length1, int* result_length1);
static gpointer* gee_array_list_wrap_ulong (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gulong* data, int data_length1, int* result_length1);
static gpointer* gee_array_list_wrap_float (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gfloat** data, int data_length1, int* result_length1);
static gpointer* gee_array_list_wrap_double (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gdouble** data, int data_length1, int* result_length1);
static void gee_array_list_set_equal_func (GeeArrayList* self, GeeEqualDataFunc value, gpointer value_target);
#define GEE_ARRAY_LIST_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_ARRAY_LIST_TYPE_ITERATOR, GeeArrayListIteratorPrivate))
enum  {
	GEE_ARRAY_LIST_ITERATOR_DUMMY_PROPERTY,
	GEE_ARRAY_LIST_ITERATOR_G_TYPE,
	GEE_ARRAY_LIST_ITERATOR_G_DUP_FUNC,
	GEE_ARRAY_LIST_ITERATOR_G_DESTROY_FUNC,
	GEE_ARRAY_LIST_ITERATOR_READ_ONLY,
	GEE_ARRAY_LIST_ITERATOR_VALID
};
static GeeArrayListIterator* gee_array_list_iterator_new_from_iterator (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeArrayListIterator* iter);
static GeeArrayListIterator* gee_array_list_iterator_construct_from_iterator (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeArrayListIterator* iter);
static gboolean gee_array_list_iterator_real_next (GeeIterator* base);
static gboolean gee_array_list_iterator_real_has_next (GeeIterator* base);
static gboolean gee_array_list_iterator_real_first (GeeBidirIterator* base);
gint gee_abstract_collection_get_size (GeeAbstractCollection* self);
static gpointer gee_array_list_iterator_real_get (GeeIterator* base);
static void gee_array_list_iterator_real_remove (GeeIterator* base);
static gboolean gee_array_list_iterator_real_previous (GeeBidirIterator* base);
static gboolean gee_array_list_iterator_real_has_previous (GeeBidirIterator* base);
static gboolean gee_array_list_iterator_real_last (GeeBidirIterator* base);
static void gee_array_list_iterator_real_set (GeeListIterator* base, gconstpointer item);
static void gee_array_list_iterator_real_insert (GeeBidirListIterator* base, gconstpointer item);
void gee_abstract_list_insert (GeeAbstractList* self, gint index, gconstpointer item);
static void gee_array_list_iterator_real_add (GeeListIterator* base, gconstpointer item);
static gint gee_array_list_iterator_real_index (GeeListIterator* base);
static gboolean gee_array_list_iterator_real_foreach (GeeTraversable* base, GeeForallFunc f, void* f_target);
static GeeIterator** gee_array_list_iterator_real_tee (GeeTraversable* base, guint forks, int* result_length1);
static void gee_array_list_iterator_finalize (GObject* obj);
gboolean gee_iterator_get_read_only (GeeIterator* self);
gboolean gee_iterator_get_valid (GeeIterator* self);
static void _vala_gee_array_list_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gee_array_list_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void gee_array_list_finalize (GObject* obj);
gboolean gee_abstract_collection_get_read_only (GeeAbstractCollection* self);
static void _vala_gee_array_list_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gee_array_list_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_move (gpointer array, gsize element_size, gint src, gint dest, gint length);


/**
 * Constructs a new, empty array list.
 *
 * If not provided, the function parameter is requested to the
 * {@link Functions} function factory methods.
 *
 * @param equal_func an optional element equality testing function
 */
GeeArrayList* gee_array_list_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeEqualDataFunc equal_func, void* equal_func_target, GDestroyNotify equal_func_target_destroy_notify) {
	GeeArrayList * self = NULL;
	GeeEqualDataFunc _tmp0_ = NULL;
	void* _tmp0__target = NULL;
	GeeEqualDataFunc _tmp4_ = NULL;
	void* _tmp4__target = NULL;
	GDestroyNotify _tmp4__target_destroy_notify = NULL;
	GeeFunctionsEqualDataFuncClosure* _tmp5_ = NULL;
	gpointer* _tmp6_ = NULL;
	self = (GeeArrayList*) gee_abstract_bidir_list_construct (object_type, g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	_tmp0_ = equal_func;
	_tmp0__target = equal_func_target;
	if (_tmp0_ == NULL) {
		void* _tmp1_ = NULL;
		GDestroyNotify _tmp2_ = NULL;
		GeeEqualDataFunc _tmp3_ = NULL;
		_tmp3_ = gee_functions_get_equal_func_for (g_type, &_tmp1_, &_tmp2_);
		(equal_func_target_destroy_notify == NULL) ? NULL : (equal_func_target_destroy_notify (equal_func_target), NULL);
		equal_func = NULL;
		equal_func_target = NULL;
		equal_func_target_destroy_notify = NULL;
		equal_func = _tmp3_;
		equal_func_target = _tmp1_;
		equal_func_target_destroy_notify = _tmp2_;
	}
	_tmp4_ = equal_func;
	_tmp4__target = equal_func_target;
	_tmp4__target_destroy_notify = equal_func_target_destroy_notify;
	equal_func_target_destroy_notify = NULL;
	_tmp5_ = gee_functions_equal_data_func_closure_new (g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func, _tmp4_, _tmp4__target, _tmp4__target_destroy_notify);
	_gee_functions_equal_data_func_closure_unref0 (self->priv->_equal_func);
	self->priv->_equal_func = _tmp5_;
	_tmp6_ = g_new0 (gpointer, 4);
	self->_items = (_vala_array_free (self->_items, self->_items_length1, (GDestroyNotify) g_destroy_func), NULL);
	self->_items = _tmp6_;
	self->_items_length1 = 4;
	self->__items_size_ = self->_items_length1;
	self->_size = 0;
	(equal_func_target_destroy_notify == NULL) ? NULL : (equal_func_target_destroy_notify (equal_func_target), NULL);
	equal_func = NULL;
	equal_func_target = NULL;
	equal_func_target_destroy_notify = NULL;
	return self;
}


GeeArrayList* gee_array_list_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeEqualDataFunc equal_func, void* equal_func_target, GDestroyNotify equal_func_target_destroy_notify) {
	return gee_array_list_construct (GEE_TYPE_ARRAY_LIST, g_type, g_dup_func, g_destroy_func, equal_func, equal_func_target, equal_func_target_destroy_notify);
}


/**
 * Constructs a new array list based on provided array.
 *
 * If not provided, the function parameter is requested to the
 * {@link Functions} function factory methods.
 *
 * @param items initial items to be put into array
 * @param equal_func an optional element equality testing function
 */
GeeArrayList* gee_array_list_construct_wrap (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gpointer* items, int items_length1, GeeEqualDataFunc equal_func, void* equal_func_target, GDestroyNotify equal_func_target_destroy_notify) {
	GeeArrayList * self = NULL;
	GeeEqualDataFunc _tmp0_ = NULL;
	void* _tmp0__target = NULL;
	GeeEqualDataFunc _tmp4_ = NULL;
	void* _tmp4__target = NULL;
	GDestroyNotify _tmp4__target_destroy_notify = NULL;
	GeeFunctionsEqualDataFuncClosure* _tmp5_ = NULL;
	gpointer* _tmp6_ = NULL;
	gint _tmp6__length1 = 0;
	gpointer* _tmp7_ = NULL;
	gint _tmp7__length1 = 0;
	gint _tmp8_ = 0;
	gpointer* _tmp9_ = NULL;
	self = (GeeArrayList*) gee_abstract_bidir_list_construct (object_type, g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	_tmp0_ = equal_func;
	_tmp0__target = equal_func_target;
	if (_tmp0_ == NULL) {
		void* _tmp1_ = NULL;
		GDestroyNotify _tmp2_ = NULL;
		GeeEqualDataFunc _tmp3_ = NULL;
		_tmp3_ = gee_functions_get_equal_func_for (g_type, &_tmp1_, &_tmp2_);
		(equal_func_target_destroy_notify == NULL) ? NULL : (equal_func_target_destroy_notify (equal_func_target), NULL);
		equal_func = NULL;
		equal_func_target = NULL;
		equal_func_target_destroy_notify = NULL;
		equal_func = _tmp3_;
		equal_func_target = _tmp1_;
		equal_func_target_destroy_notify = _tmp2_;
	}
	_tmp4_ = equal_func;
	_tmp4__target = equal_func_target;
	_tmp4__target_destroy_notify = equal_func_target_destroy_notify;
	equal_func_target_destroy_notify = NULL;
	_tmp5_ = gee_functions_equal_data_func_closure_new (g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func, _tmp4_, _tmp4__target, _tmp4__target_destroy_notify);
	_gee_functions_equal_data_func_closure_unref0 (self->priv->_equal_func);
	self->priv->_equal_func = _tmp5_;
	_tmp6_ = items;
	_tmp6__length1 = items_length1;
	self->_size = _tmp6__length1;
	_tmp7_ = items;
	_tmp7__length1 = items_length1;
	items = NULL;
	items_length1 = 0;
	_tmp9_ = gee_array_list_do_wrap (g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func, _tmp7_, _tmp7__length1, &_tmp8_);
	self->_items = (_vala_array_free (self->_items, self->_items_length1, (GDestroyNotify) g_destroy_func), NULL);
	self->_items = _tmp9_;
	self->_items_length1 = _tmp8_;
	self->__items_size_ = self->_items_length1;
	items = (_vala_array_free (items, items_length1, (GDestroyNotify) g_destroy_func), NULL);
	(equal_func_target_destroy_notify == NULL) ? NULL : (equal_func_target_destroy_notify (equal_func_target), NULL);
	equal_func = NULL;
	equal_func_target = NULL;
	equal_func_target_destroy_notify = NULL;
	return self;
}


GeeArrayList* gee_array_list_new_wrap (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gpointer* items, int items_length1, GeeEqualDataFunc equal_func, void* equal_func_target, GDestroyNotify equal_func_target_destroy_notify) {
	return gee_array_list_construct_wrap (GEE_TYPE_ARRAY_LIST, g_type, g_dup_func, g_destroy_func, items, items_length1, equal_func, equal_func_target, equal_func_target_destroy_notify);
}


static gpointer _gee_functions_equal_data_func_closure_ref0 (gpointer self) {
	return self ? gee_functions_equal_data_func_closure_ref (self) : NULL;
}


G_GNUC_INTERNAL GeeArrayList* gee_array_list_construct_with_closure (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeFunctionsEqualDataFuncClosure* equal_func) {
	GeeArrayList * self = NULL;
	GeeFunctionsEqualDataFuncClosure* _tmp0_ = NULL;
	GeeFunctionsEqualDataFuncClosure* _tmp1_ = NULL;
	gpointer* _tmp2_ = NULL;
	g_return_val_if_fail (equal_func != NULL, NULL);
	self = (GeeArrayList*) gee_abstract_bidir_list_construct (object_type, g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	_tmp0_ = equal_func;
	_tmp1_ = _gee_functions_equal_data_func_closure_ref0 (_tmp0_);
	_gee_functions_equal_data_func_closure_unref0 (self->priv->_equal_func);
	self->priv->_equal_func = _tmp1_;
	_tmp2_ = g_new0 (gpointer, 4);
	self->_items = (_vala_array_free (self->_items, self->_items_length1, (GDestroyNotify) g_destroy_func), NULL);
	self->_items = _tmp2_;
	self->_items_length1 = 4;
	self->__items_size_ = self->_items_length1;
	self->_size = 0;
	_gee_functions_equal_data_func_closure_unref0 (equal_func);
	return self;
}


G_GNUC_INTERNAL GeeArrayList* gee_array_list_new_with_closure (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeFunctionsEqualDataFuncClosure* equal_func) {
	return gee_array_list_construct_with_closure (GEE_TYPE_ARRAY_LIST, g_type, g_dup_func, g_destroy_func, equal_func);
}


/**
 * {@inheritDoc}
 */
static gboolean gee_array_list_real_foreach (GeeAbstractCollection* base, GeeForallFunc f, void* f_target) {
	GeeArrayList * self;
	gboolean result = FALSE;
	self = (GeeArrayList*) base;
	{
		gint i = 0;
		i = 0;
		{
			gboolean _tmp0_ = FALSE;
			_tmp0_ = TRUE;
			while (TRUE) {
				gint _tmp2_ = 0;
				gint _tmp3_ = 0;
				GeeForallFunc _tmp4_ = NULL;
				void* _tmp4__target = NULL;
				gpointer* _tmp5_ = NULL;
				gint _tmp5__length1 = 0;
				gint _tmp6_ = 0;
				gconstpointer _tmp7_ = NULL;
				gpointer _tmp8_ = NULL;
				gboolean _tmp9_ = FALSE;
				if (!_tmp0_) {
					gint _tmp1_ = 0;
					_tmp1_ = i;
					i = _tmp1_ + 1;
				}
				_tmp0_ = FALSE;
				_tmp2_ = i;
				_tmp3_ = self->_size;
				if (!(_tmp2_ < _tmp3_)) {
					break;
				}
				_tmp4_ = f;
				_tmp4__target = f_target;
				_tmp5_ = self->_items;
				_tmp5__length1 = self->_items_length1;
				_tmp6_ = i;
				_tmp7_ = _tmp5_[_tmp6_];
				_tmp8_ = ((_tmp7_ != NULL) && (self->priv->g_dup_func != NULL)) ? self->priv->g_dup_func ((gpointer) _tmp7_) : ((gpointer) _tmp7_);
				_tmp9_ = _tmp4_ (_tmp8_, _tmp4__target);
				if (!_tmp9_) {
					result = FALSE;
					return result;
				}
			}
		}
	}
	result = TRUE;
	return result;
}


/**
 * {@inheritDoc}
 */
static GeeIterator* gee_array_list_real_iterator (GeeAbstractCollection* base) {
	GeeArrayList * self;
	GeeIterator* result = NULL;
	GeeArrayListIterator* _tmp0_ = NULL;
	self = (GeeArrayList*) base;
	_tmp0_ = gee_array_list_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, self);
	result = (GeeIterator*) _tmp0_;
	return result;
}


/**
 * {@inheritDoc}
 */
static GeeListIterator* gee_array_list_real_list_iterator (GeeAbstractList* base) {
	GeeArrayList * self;
	GeeListIterator* result = NULL;
	GeeArrayListIterator* _tmp0_ = NULL;
	self = (GeeArrayList*) base;
	_tmp0_ = gee_array_list_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, self);
	result = (GeeListIterator*) _tmp0_;
	return result;
}


/**
 * {@inheritDoc}
 */
static GeeBidirListIterator* gee_array_list_real_bidir_list_iterator (GeeAbstractBidirList* base) {
	GeeArrayList * self;
	GeeBidirListIterator* result = NULL;
	GeeArrayListIterator* _tmp0_ = NULL;
	self = (GeeArrayList*) base;
	_tmp0_ = gee_array_list_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, self);
	result = (GeeBidirListIterator*) _tmp0_;
	return result;
}


/**
 * {@inheritDoc}
 */
static gboolean gee_array_list_real_contains (GeeAbstractCollection* base, gconstpointer item) {
	GeeArrayList * self;
	gboolean result = FALSE;
	gconstpointer _tmp0_ = NULL;
	gint _tmp1_ = 0;
	self = (GeeArrayList*) base;
	_tmp0_ = item;
	_tmp1_ = gee_abstract_list_index_of ((GeeAbstractList*) self, _tmp0_);
	result = _tmp1_ != (-1);
	return result;
}


/**
 * {@inheritDoc}
 */
static gint gee_array_list_real_index_of (GeeAbstractList* base, gconstpointer item) {
	GeeArrayList * self;
	gint result = 0;
	self = (GeeArrayList*) base;
	{
		gint index = 0;
		index = 0;
		{
			gboolean _tmp0_ = FALSE;
			_tmp0_ = TRUE;
			while (TRUE) {
				gint _tmp2_ = 0;
				gint _tmp3_ = 0;
				GeeEqualDataFunc _tmp4_ = NULL;
				void* _tmp4__target = NULL;
				GeeEqualDataFunc _tmp5_ = NULL;
				void* _tmp5__target = NULL;
				gpointer* _tmp6_ = NULL;
				gint _tmp6__length1 = 0;
				gint _tmp7_ = 0;
				gconstpointer _tmp8_ = NULL;
				gconstpointer _tmp9_ = NULL;
				gboolean _tmp10_ = FALSE;
				if (!_tmp0_) {
					gint _tmp1_ = 0;
					_tmp1_ = index;
					index = _tmp1_ + 1;
				}
				_tmp0_ = FALSE;
				_tmp2_ = index;
				_tmp3_ = self->_size;
				if (!(_tmp2_ < _tmp3_)) {
					break;
				}
				_tmp4_ = gee_array_list_get_equal_func (self, &_tmp4__target);
				_tmp5_ = _tmp4_;
				_tmp5__target = _tmp4__target;
				_tmp6_ = self->_items;
				_tmp6__length1 = self->_items_length1;
				_tmp7_ = index;
				_tmp8_ = _tmp6_[_tmp7_];
				_tmp9_ = item;
				_tmp10_ = _tmp5_ (_tmp8_, _tmp9_, _tmp5__target);
				if (_tmp10_) {
					result = index;
					return result;
				}
			}
		}
	}
	result = -1;
	return result;
}


/**
 * {@inheritDoc}
 */
static gpointer gee_array_list_real_get (GeeAbstractList* base, gint index) {
	GeeArrayList * self;
	gpointer result = NULL;
	gint _tmp0_ = 0;
	gint _tmp1_ = 0;
	gint _tmp2_ = 0;
	gpointer* _tmp3_ = NULL;
	gint _tmp3__length1 = 0;
	gint _tmp4_ = 0;
	gconstpointer _tmp5_ = NULL;
	gpointer _tmp6_ = NULL;
	self = (GeeArrayList*) base;
	_tmp0_ = index;
	_vala_assert (_tmp0_ >= 0, "index >= 0");
	_tmp1_ = index;
	_tmp2_ = self->_size;
	_vala_assert (_tmp1_ < _tmp2_, "index < _size");
	_tmp3_ = self->_items;
	_tmp3__length1 = self->_items_length1;
	_tmp4_ = index;
	_tmp5_ = _tmp3_[_tmp4_];
	_tmp6_ = ((_tmp5_ != NULL) && (self->priv->g_dup_func != NULL)) ? self->priv->g_dup_func ((gpointer) _tmp5_) : ((gpointer) _tmp5_);
	result = _tmp6_;
	return result;
}


/**
 * {@inheritDoc}
 */
static void gee_array_list_real_set (GeeAbstractList* base, gint index, gconstpointer item) {
	GeeArrayList * self;
	gint _tmp0_ = 0;
	gint _tmp1_ = 0;
	gint _tmp2_ = 0;
	gpointer* _tmp3_ = NULL;
	gint _tmp3__length1 = 0;
	gint _tmp4_ = 0;
	gconstpointer _tmp5_ = NULL;
	gpointer _tmp6_ = NULL;
	gpointer _tmp7_ = NULL;
	self = (GeeArrayList*) base;
	_tmp0_ = index;
	_vala_assert (_tmp0_ >= 0, "index >= 0");
	_tmp1_ = index;
	_tmp2_ = self->_size;
	_vala_assert (_tmp1_ < _tmp2_, "index < _size");
	_tmp3_ = self->_items;
	_tmp3__length1 = self->_items_length1;
	_tmp4_ = index;
	_tmp5_ = item;
	_tmp6_ = ((_tmp5_ != NULL) && (self->priv->g_dup_func != NULL)) ? self->priv->g_dup_func ((gpointer) _tmp5_) : ((gpointer) _tmp5_);
	((_tmp3_[_tmp4_] == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (_tmp3_[_tmp4_] = (self->priv->g_destroy_func (_tmp3_[_tmp4_]), NULL));
	_tmp3_[_tmp4_] = _tmp6_;
	_tmp7_ = _tmp3_[_tmp4_];
}


/**
 * {@inheritDoc}
 */
static gboolean gee_array_list_real_add (GeeAbstractCollection* base, gconstpointer item) {
	GeeArrayList * self;
	gboolean result = FALSE;
	gint _tmp0_ = 0;
	gpointer* _tmp1_ = NULL;
	gint _tmp1__length1 = 0;
	gpointer* _tmp2_ = NULL;
	gint _tmp2__length1 = 0;
	gint _tmp3_ = 0;
	gconstpointer _tmp4_ = NULL;
	gpointer _tmp5_ = NULL;
	gpointer _tmp6_ = NULL;
	gint _tmp7_ = 0;
	self = (GeeArrayList*) base;
	_tmp0_ = self->_size;
	_tmp1_ = self->_items;
	_tmp1__length1 = self->_items_length1;
	if (_tmp0_ == _tmp1__length1) {
		gee_array_list_grow_if_needed (self, 1);
	}
	_tmp2_ = self->_items;
	_tmp2__length1 = self->_items_length1;
	_tmp3_ = self->_size;
	self->_size = _tmp3_ + 1;
	_tmp4_ = item;
	_tmp5_ = ((_tmp4_ != NULL) && (self->priv->g_dup_func != NULL)) ? self->priv->g_dup_func ((gpointer) _tmp4_) : ((gpointer) _tmp4_);
	((_tmp2_[_tmp3_] == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (_tmp2_[_tmp3_] = (self->priv->g_destroy_func (_tmp2_[_tmp3_]), NULL));
	_tmp2_[_tmp3_] = _tmp5_;
	_tmp6_ = _tmp2_[_tmp3_];
	_tmp7_ = self->priv->_stamp;
	self->priv->_stamp = _tmp7_ + 1;
	result = TRUE;
	return result;
}


/**
 * {@inheritDoc}
 */
static void gee_array_list_real_insert (GeeAbstractList* base, gint index, gconstpointer item) {
	GeeArrayList * self;
	gint _tmp0_ = 0;
	gint _tmp1_ = 0;
	gint _tmp2_ = 0;
	gint _tmp3_ = 0;
	gpointer* _tmp4_ = NULL;
	gint _tmp4__length1 = 0;
	gint _tmp5_ = 0;
	gpointer* _tmp6_ = NULL;
	gint _tmp6__length1 = 0;
	gint _tmp7_ = 0;
	gconstpointer _tmp8_ = NULL;
	gpointer _tmp9_ = NULL;
	gpointer _tmp10_ = NULL;
	gint _tmp11_ = 0;
	self = (GeeArrayList*) base;
	_tmp0_ = index;
	_vala_assert (_tmp0_ >= 0, "index >= 0");
	_tmp1_ = index;
	_tmp2_ = self->_size;
	_vala_assert (_tmp1_ <= _tmp2_, "index <= _size");
	_tmp3_ = self->_size;
	_tmp4_ = self->_items;
	_tmp4__length1 = self->_items_length1;
	if (_tmp3_ == _tmp4__length1) {
		gee_array_list_grow_if_needed (self, 1);
	}
	_tmp5_ = index;
	gee_array_list_shift (self, _tmp5_, 1);
	_tmp6_ = self->_items;
	_tmp6__length1 = self->_items_length1;
	_tmp7_ = index;
	_tmp8_ = item;
	_tmp9_ = ((_tmp8_ != NULL) && (self->priv->g_dup_func != NULL)) ? self->priv->g_dup_func ((gpointer) _tmp8_) : ((gpointer) _tmp8_);
	((_tmp6_[_tmp7_] == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (_tmp6_[_tmp7_] = (self->priv->g_destroy_func (_tmp6_[_tmp7_]), NULL));
	_tmp6_[_tmp7_] = _tmp9_;
	_tmp10_ = _tmp6_[_tmp7_];
	_tmp11_ = self->priv->_stamp;
	self->priv->_stamp = _tmp11_ + 1;
}


/**
 * {@inheritDoc}
 */
static gboolean gee_array_list_real_remove (GeeAbstractCollection* base, gconstpointer item) {
	GeeArrayList * self;
	gboolean result = FALSE;
	self = (GeeArrayList*) base;
	{
		gint index = 0;
		index = 0;
		{
			gboolean _tmp0_ = FALSE;
			_tmp0_ = TRUE;
			while (TRUE) {
				gint _tmp2_ = 0;
				gint _tmp3_ = 0;
				GeeEqualDataFunc _tmp4_ = NULL;
				void* _tmp4__target = NULL;
				GeeEqualDataFunc _tmp5_ = NULL;
				void* _tmp5__target = NULL;
				gpointer* _tmp6_ = NULL;
				gint _tmp6__length1 = 0;
				gint _tmp7_ = 0;
				gconstpointer _tmp8_ = NULL;
				gconstpointer _tmp9_ = NULL;
				gboolean _tmp10_ = FALSE;
				if (!_tmp0_) {
					gint _tmp1_ = 0;
					_tmp1_ = index;
					index = _tmp1_ + 1;
				}
				_tmp0_ = FALSE;
				_tmp2_ = index;
				_tmp3_ = self->_size;
				if (!(_tmp2_ < _tmp3_)) {
					break;
				}
				_tmp4_ = gee_array_list_get_equal_func (self, &_tmp4__target);
				_tmp5_ = _tmp4_;
				_tmp5__target = _tmp4__target;
				_tmp6_ = self->_items;
				_tmp6__length1 = self->_items_length1;
				_tmp7_ = index;
				_tmp8_ = _tmp6_[_tmp7_];
				_tmp9_ = item;
				_tmp10_ = _tmp5_ (_tmp8_, _tmp9_, _tmp5__target);
				if (_tmp10_) {
					gint _tmp11_ = 0;
					gpointer _tmp12_ = NULL;
					gpointer _tmp13_ = NULL;
					_tmp11_ = index;
					_tmp12_ = gee_abstract_list_remove_at ((GeeAbstractList*) self, _tmp11_);
					_tmp13_ = _tmp12_;
					((_tmp13_ == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (_tmp13_ = (self->priv->g_destroy_func (_tmp13_), NULL));
					result = TRUE;
					return result;
				}
			}
		}
	}
	result = FALSE;
	return result;
}


/**
 * {@inheritDoc}
 */
static gpointer gee_array_list_real_remove_at (GeeAbstractList* base, gint index) {
	GeeArrayList * self;
	gpointer result = NULL;
	gint _tmp0_ = 0;
	gint _tmp1_ = 0;
	gint _tmp2_ = 0;
	gpointer item = NULL;
	gpointer* _tmp3_ = NULL;
	gint _tmp3__length1 = 0;
	gint _tmp4_ = 0;
	gconstpointer _tmp5_ = NULL;
	gpointer _tmp6_ = NULL;
	gpointer* _tmp7_ = NULL;
	gint _tmp7__length1 = 0;
	gint _tmp8_ = 0;
	gpointer _tmp9_ = NULL;
	gint _tmp10_ = 0;
	gint _tmp11_ = 0;
	self = (GeeArrayList*) base;
	_tmp0_ = index;
	_vala_assert (_tmp0_ >= 0, "index >= 0");
	_tmp1_ = index;
	_tmp2_ = self->_size;
	_vala_assert (_tmp1_ < _tmp2_, "index < _size");
	_tmp3_ = self->_items;
	_tmp3__length1 = self->_items_length1;
	_tmp4_ = index;
	_tmp5_ = _tmp3_[_tmp4_];
	_tmp6_ = ((_tmp5_ != NULL) && (self->priv->g_dup_func != NULL)) ? self->priv->g_dup_func ((gpointer) _tmp5_) : ((gpointer) _tmp5_);
	item = _tmp6_;
	_tmp7_ = self->_items;
	_tmp7__length1 = self->_items_length1;
	_tmp8_ = index;
	((_tmp7_[_tmp8_] == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (_tmp7_[_tmp8_] = (self->priv->g_destroy_func (_tmp7_[_tmp8_]), NULL));
	_tmp7_[_tmp8_] = NULL;
	_tmp9_ = _tmp7_[_tmp8_];
	_tmp10_ = index;
	gee_array_list_shift (self, _tmp10_ + 1, -1);
	_tmp11_ = self->priv->_stamp;
	self->priv->_stamp = _tmp11_ + 1;
	result = item;
	return result;
}


/**
 * {@inheritDoc}
 */
static void gee_array_list_real_clear (GeeAbstractCollection* base) {
	GeeArrayList * self;
	gint _tmp7_ = 0;
	self = (GeeArrayList*) base;
	{
		gint index = 0;
		index = 0;
		{
			gboolean _tmp0_ = FALSE;
			_tmp0_ = TRUE;
			while (TRUE) {
				gint _tmp2_ = 0;
				gint _tmp3_ = 0;
				gpointer* _tmp4_ = NULL;
				gint _tmp4__length1 = 0;
				gint _tmp5_ = 0;
				gpointer _tmp6_ = NULL;
				if (!_tmp0_) {
					gint _tmp1_ = 0;
					_tmp1_ = index;
					index = _tmp1_ + 1;
				}
				_tmp0_ = FALSE;
				_tmp2_ = index;
				_tmp3_ = self->_size;
				if (!(_tmp2_ < _tmp3_)) {
					break;
				}
				_tmp4_ = self->_items;
				_tmp4__length1 = self->_items_length1;
				_tmp5_ = index;
				((_tmp4_[_tmp5_] == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (_tmp4_[_tmp5_] = (self->priv->g_destroy_func (_tmp4_[_tmp5_]), NULL));
				_tmp4_[_tmp5_] = NULL;
				_tmp6_ = _tmp4_[_tmp5_];
			}
		}
	}
	self->_size = 0;
	_tmp7_ = self->priv->_stamp;
	self->priv->_stamp = _tmp7_ + 1;
}


/**
 * {@inheritDoc}
 */
static GeeList* gee_array_list_real_slice (GeeAbstractList* base, gint start, gint stop) {
	GeeArrayList * self;
	GeeList* result = NULL;
	gint _tmp0_ = 0;
	gint _tmp1_ = 0;
	gint _tmp2_ = 0;
	gint _tmp3_ = 0;
	gint _tmp4_ = 0;
	GeeArrayList* slice = NULL;
	GeeFunctionsEqualDataFuncClosure* _tmp5_ = NULL;
	GeeFunctionsEqualDataFuncClosure* _tmp6_ = NULL;
	GeeArrayList* _tmp7_ = NULL;
	self = (GeeArrayList*) base;
	_tmp0_ = start;
	_tmp1_ = stop;
	g_return_val_if_fail (_tmp0_ <= _tmp1_, NULL);
	_tmp2_ = start;
	g_return_val_if_fail (_tmp2_ >= 0, NULL);
	_tmp3_ = stop;
	_tmp4_ = self->_size;
	g_return_val_if_fail (_tmp3_ <= _tmp4_, NULL);
	_tmp5_ = self->priv->_equal_func;
	_tmp6_ = _gee_functions_equal_data_func_closure_ref0 (_tmp5_);
	_tmp7_ = gee_array_list_new_with_closure (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, _tmp6_);
	slice = _tmp7_;
	{
		gint i = 0;
		gint _tmp8_ = 0;
		_tmp8_ = start;
		i = _tmp8_;
		{
			gboolean _tmp9_ = FALSE;
			_tmp9_ = TRUE;
			while (TRUE) {
				gint _tmp11_ = 0;
				gint _tmp12_ = 0;
				GeeArrayList* _tmp13_ = NULL;
				gint _tmp14_ = 0;
				gpointer _tmp15_ = NULL;
				gpointer _tmp16_ = NULL;
				if (!_tmp9_) {
					gint _tmp10_ = 0;
					_tmp10_ = i;
					i = _tmp10_ + 1;
				}
				_tmp9_ = FALSE;
				_tmp11_ = i;
				_tmp12_ = stop;
				if (!(_tmp11_ < _tmp12_)) {
					break;
				}
				_tmp13_ = slice;
				_tmp14_ = i;
				_tmp15_ = gee_abstract_list_get ((GeeAbstractList*) self, _tmp14_);
				_tmp16_ = _tmp15_;
				gee_abstract_collection_add ((GeeAbstractCollection*) _tmp13_, _tmp16_);
				((_tmp16_ == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (_tmp16_ = (self->priv->g_destroy_func (_tmp16_), NULL));
			}
		}
	}
	result = (GeeList*) slice;
	return result;
}


/**
 * {@inheritDoc}
 */
static gboolean __lambda43_ (GeeArrayList* self, gpointer item) {
	gboolean result = FALSE;
	gpointer* _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
	gint _tmp1_ = 0;
	gconstpointer _tmp2_ = NULL;
	gpointer _tmp3_ = NULL;
	gpointer _tmp4_ = NULL;
	_tmp0_ = self->_items;
	_tmp0__length1 = self->_items_length1;
	_tmp1_ = self->_size;
	self->_size = _tmp1_ + 1;
	_tmp2_ = item;
	_tmp3_ = ((_tmp2_ != NULL) && (self->priv->g_dup_func != NULL)) ? self->priv->g_dup_func ((gpointer) _tmp2_) : ((gpointer) _tmp2_);
	((_tmp0_[_tmp1_] == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (_tmp0_[_tmp1_] = (self->priv->g_destroy_func (_tmp0_[_tmp1_]), NULL));
	_tmp0_[_tmp1_] = _tmp3_;
	_tmp4_ = _tmp0_[_tmp1_];
	result = TRUE;
	((item == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (item = (self->priv->g_destroy_func (item), NULL));
	return result;
}


static gboolean ___lambda43__gee_forall_func (gpointer g, gpointer self) {
	gboolean result;
	result = __lambda43_ ((GeeArrayList*) self, g);
	return result;
}


gboolean gee_array_list_add_all (GeeArrayList* self, GeeCollection* collection) {
	gboolean result = FALSE;
	GeeCollection* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
	gboolean _tmp2_ = FALSE;
	GeeCollection* _tmp3_ = NULL;
	gint _tmp4_ = 0;
	gint _tmp5_ = 0;
	GeeCollection* _tmp6_ = NULL;
	gint _tmp7_ = 0;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (collection != NULL, FALSE);
	_tmp0_ = collection;
	_tmp1_ = gee_collection_get_is_empty (_tmp0_);
	_tmp2_ = _tmp1_;
	if (_tmp2_) {
		result = FALSE;
		return result;
	}
	_tmp3_ = collection;
	_tmp4_ = gee_collection_get_size (_tmp3_);
	_tmp5_ = _tmp4_;
	gee_array_list_grow_if_needed (self, _tmp5_);
	_tmp6_ = collection;
	gee_traversable_foreach ((GeeTraversable*) _tmp6_, ___lambda43__gee_forall_func, self);
	_tmp7_ = self->priv->_stamp;
	self->priv->_stamp = _tmp7_ + 1;
	result = TRUE;
	return result;
}


static void gee_array_list_shift (GeeArrayList* self, gint start, gint delta) {
	gint _tmp0_ = 0;
	gint _tmp1_ = 0;
	gint _tmp2_ = 0;
	gint _tmp3_ = 0;
	gint _tmp4_ = 0;
	gint _tmp5_ = 0;
	gint _tmp6_ = 0;
	gint _tmp7_ = 0;
	gint _tmp8_ = 0;
	gint _tmp9_ = 0;
	gint _tmp10_ = 0;
	gint _tmp11_ = 0;
	g_return_if_fail (self != NULL);
	_tmp0_ = start;
	_vala_assert (_tmp0_ >= 0, "start >= 0");
	_tmp1_ = start;
	_tmp2_ = self->_size;
	_vala_assert (_tmp1_ <= _tmp2_, "start <= _size");
	_tmp3_ = start;
	_tmp4_ = delta;
	_vala_assert (_tmp3_ >= (-_tmp4_), "start >= -delta");
	_tmp5_ = start;
	_tmp6_ = start;
	_tmp7_ = delta;
	_tmp8_ = self->_size;
	_tmp9_ = start;
	_vala_array_move (self->_items, sizeof (gpointer), _tmp5_, _tmp6_ + _tmp7_, _tmp8_ - _tmp9_);
	_tmp10_ = self->_size;
	_tmp11_ = delta;
	self->_size = _tmp10_ + _tmp11_;
}


static void gee_array_list_grow_if_needed (GeeArrayList* self, gint new_count) {
	gint _tmp0_ = 0;
	gint minimum_size = 0;
	gint _tmp1_ = 0;
	gint _tmp2_ = 0;
	gint _tmp3_ = 0;
	gpointer* _tmp4_ = NULL;
	gint _tmp4__length1 = 0;
	g_return_if_fail (self != NULL);
	_tmp0_ = new_count;
	_vala_assert (_tmp0_ >= 0, "new_count >= 0");
	_tmp1_ = self->_size;
	_tmp2_ = new_count;
	minimum_size = _tmp1_ + _tmp2_;
	_tmp3_ = minimum_size;
	_tmp4_ = self->_items;
	_tmp4__length1 = self->_items_length1;
	if (_tmp3_ > _tmp4__length1) {
		gint _tmp5_ = 0;
		gint _tmp6_ = 0;
		gpointer* _tmp7_ = NULL;
		gint _tmp7__length1 = 0;
		_tmp6_ = new_count;
		_tmp7_ = self->_items;
		_tmp7__length1 = self->_items_length1;
		if (_tmp6_ > _tmp7__length1) {
			gint _tmp8_ = 0;
			_tmp8_ = minimum_size;
			_tmp5_ = _tmp8_;
		} else {
			gpointer* _tmp9_ = NULL;
			gint _tmp9__length1 = 0;
			_tmp9_ = self->_items;
			_tmp9__length1 = self->_items_length1;
			_tmp5_ = 2 * _tmp9__length1;
		}
		gee_array_list_set_capacity (self, _tmp5_);
	}
}


static void gee_array_list_set_capacity (GeeArrayList* self, gint value) {
	gint _tmp0_ = 0;
	gint _tmp1_ = 0;
	gint _tmp2_ = 0;
	gint _tmp3_ = 0;
	g_return_if_fail (self != NULL);
	_tmp0_ = value;
	_tmp1_ = self->_size;
	_vala_assert (_tmp0_ >= _tmp1_, "value >= _size");
	_tmp2_ = value;
	_tmp3_ = _tmp2_;
	self->_items = g_renew (gpointer, self->_items, _tmp2_);
	(_tmp3_ > self->_items_length1) ? memset (self->_items + self->_items_length1, 0, sizeof (gpointer) * (_tmp3_ - self->_items_length1)) : NULL;
	self->_items_length1 = _tmp3_;
	self->__items_size_ = _tmp3_;
}


static gpointer* gee_array_list_do_wrap (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gpointer* data, int data_length1, int* result_length1) {
	gpointer* result = NULL;
	GType t = 0UL;
	GType _tmp0_ = 0UL;
	t = g_type;
	_tmp0_ = t;
	if (_tmp0_ == G_TYPE_BOOLEAN) {
		gpointer* _tmp1_ = NULL;
		gint _tmp1__length1 = 0;
		gint _tmp2_ = 0;
		gpointer* _tmp3_ = NULL;
		gpointer* _tmp4_ = NULL;
		gint _tmp4__length1 = 0;
		_tmp1_ = data;
		_tmp1__length1 = data_length1;
		_tmp3_ = gee_array_list_wrap_bool (g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func, (gboolean*) _tmp1_, _tmp1__length1, &_tmp2_);
		_tmp4_ = _tmp3_;
		_tmp4__length1 = _tmp2_;
		if (result_length1) {
			*result_length1 = _tmp4__length1;
		}
		result = _tmp4_;
		data = (_vala_array_free (data, data_length1, (GDestroyNotify) g_destroy_func), NULL);
		return result;
	} else {
		GType _tmp5_ = 0UL;
		_tmp5_ = t;
		if (_tmp5_ == G_TYPE_CHAR) {
			gpointer* _tmp6_ = NULL;
			gint _tmp6__length1 = 0;
			gint _tmp7_ = 0;
			gpointer* _tmp8_ = NULL;
			gpointer* _tmp9_ = NULL;
			gint _tmp9__length1 = 0;
			_tmp6_ = data;
			_tmp6__length1 = data_length1;
			_tmp8_ = gee_array_list_wrap_char (g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func, (gchar*) _tmp6_, _tmp6__length1, &_tmp7_);
			_tmp9_ = _tmp8_;
			_tmp9__length1 = _tmp7_;
			if (result_length1) {
				*result_length1 = _tmp9__length1;
			}
			result = _tmp9_;
			data = (_vala_array_free (data, data_length1, (GDestroyNotify) g_destroy_func), NULL);
			return result;
		} else {
			GType _tmp10_ = 0UL;
			_tmp10_ = t;
			if (_tmp10_ == G_TYPE_UCHAR) {
				gpointer* _tmp11_ = NULL;
				gint _tmp11__length1 = 0;
				gint _tmp12_ = 0;
				gpointer* _tmp13_ = NULL;
				gpointer* _tmp14_ = NULL;
				gint _tmp14__length1 = 0;
				_tmp11_ = data;
				_tmp11__length1 = data_length1;
				_tmp13_ = gee_array_list_wrap_uchar (g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func, (guchar*) _tmp11_, _tmp11__length1, &_tmp12_);
				_tmp14_ = _tmp13_;
				_tmp14__length1 = _tmp12_;
				if (result_length1) {
					*result_length1 = _tmp14__length1;
				}
				result = _tmp14_;
				data = (_vala_array_free (data, data_length1, (GDestroyNotify) g_destroy_func), NULL);
				return result;
			} else {
				GType _tmp15_ = 0UL;
				_tmp15_ = t;
				if (_tmp15_ == G_TYPE_INT) {
					gpointer* _tmp16_ = NULL;
					gint _tmp16__length1 = 0;
					gint _tmp17_ = 0;
					gpointer* _tmp18_ = NULL;
					gpointer* _tmp19_ = NULL;
					gint _tmp19__length1 = 0;
					_tmp16_ = data;
					_tmp16__length1 = data_length1;
					_tmp18_ = gee_array_list_wrap_int (g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func, (gint*) _tmp16_, _tmp16__length1, &_tmp17_);
					_tmp19_ = _tmp18_;
					_tmp19__length1 = _tmp17_;
					if (result_length1) {
						*result_length1 = _tmp19__length1;
					}
					result = _tmp19_;
					data = (_vala_array_free (data, data_length1, (GDestroyNotify) g_destroy_func), NULL);
					return result;
				} else {
					GType _tmp20_ = 0UL;
					_tmp20_ = t;
					if (_tmp20_ == G_TYPE_UINT) {
						gpointer* _tmp21_ = NULL;
						gint _tmp21__length1 = 0;
						gint _tmp22_ = 0;
						gpointer* _tmp23_ = NULL;
						gpointer* _tmp24_ = NULL;
						gint _tmp24__length1 = 0;
						_tmp21_ = data;
						_tmp21__length1 = data_length1;
						_tmp23_ = gee_array_list_wrap_uint (g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func, (guint*) _tmp21_, _tmp21__length1, &_tmp22_);
						_tmp24_ = _tmp23_;
						_tmp24__length1 = _tmp22_;
						if (result_length1) {
							*result_length1 = _tmp24__length1;
						}
						result = _tmp24_;
						data = (_vala_array_free (data, data_length1, (GDestroyNotify) g_destroy_func), NULL);
						return result;
					} else {
						GType _tmp25_ = 0UL;
						_tmp25_ = t;
						if (_tmp25_ == G_TYPE_INT64) {
							gpointer* _tmp26_ = NULL;
							gint _tmp26__length1 = 0;
							gint _tmp27_ = 0;
							gpointer* _tmp28_ = NULL;
							gpointer* _tmp29_ = NULL;
							gint _tmp29__length1 = 0;
							_tmp26_ = data;
							_tmp26__length1 = data_length1;
							_tmp28_ = gee_array_list_wrap_int64 (g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func, (gint64*) _tmp26_, _tmp26__length1, &_tmp27_);
							_tmp29_ = _tmp28_;
							_tmp29__length1 = _tmp27_;
							if (result_length1) {
								*result_length1 = _tmp29__length1;
							}
							result = _tmp29_;
							data = (_vala_array_free (data, data_length1, (GDestroyNotify) g_destroy_func), NULL);
							return result;
						} else {
							GType _tmp30_ = 0UL;
							_tmp30_ = t;
							if (_tmp30_ == G_TYPE_UINT64) {
								gpointer* _tmp31_ = NULL;
								gint _tmp31__length1 = 0;
								gint _tmp32_ = 0;
								gpointer* _tmp33_ = NULL;
								gpointer* _tmp34_ = NULL;
								gint _tmp34__length1 = 0;
								_tmp31_ = data;
								_tmp31__length1 = data_length1;
								_tmp33_ = gee_array_list_wrap_uint64 (g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func, (guint64*) _tmp31_, _tmp31__length1, &_tmp32_);
								_tmp34_ = _tmp33_;
								_tmp34__length1 = _tmp32_;
								if (result_length1) {
									*result_length1 = _tmp34__length1;
								}
								result = _tmp34_;
								data = (_vala_array_free (data, data_length1, (GDestroyNotify) g_destroy_func), NULL);
								return result;
							} else {
								GType _tmp35_ = 0UL;
								_tmp35_ = t;
								if (_tmp35_ == G_TYPE_LONG) {
									gpointer* _tmp36_ = NULL;
									gint _tmp36__length1 = 0;
									gint _tmp37_ = 0;
									gpointer* _tmp38_ = NULL;
									gpointer* _tmp39_ = NULL;
									gint _tmp39__length1 = 0;
									_tmp36_ = data;
									_tmp36__length1 = data_length1;
									_tmp38_ = gee_array_list_wrap_long (g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func, (glong*) _tmp36_, _tmp36__length1, &_tmp37_);
									_tmp39_ = _tmp38_;
									_tmp39__length1 = _tmp37_;
									if (result_length1) {
										*result_length1 = _tmp39__length1;
									}
									result = _tmp39_;
									data = (_vala_array_free (data, data_length1, (GDestroyNotify) g_destroy_func), NULL);
									return result;
								} else {
									GType _tmp40_ = 0UL;
									_tmp40_ = t;
									if (_tmp40_ == G_TYPE_ULONG) {
										gpointer* _tmp41_ = NULL;
										gint _tmp41__length1 = 0;
										gint _tmp42_ = 0;
										gpointer* _tmp43_ = NULL;
										gpointer* _tmp44_ = NULL;
										gint _tmp44__length1 = 0;
										_tmp41_ = data;
										_tmp41__length1 = data_length1;
										_tmp43_ = gee_array_list_wrap_ulong (g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func, (gulong*) _tmp41_, _tmp41__length1, &_tmp42_);
										_tmp44_ = _tmp43_;
										_tmp44__length1 = _tmp42_;
										if (result_length1) {
											*result_length1 = _tmp44__length1;
										}
										result = _tmp44_;
										data = (_vala_array_free (data, data_length1, (GDestroyNotify) g_destroy_func), NULL);
										return result;
									} else {
										GType _tmp45_ = 0UL;
										_tmp45_ = t;
										if (_tmp45_ == G_TYPE_FLOAT) {
											gpointer* _tmp46_ = NULL;
											gint _tmp46__length1 = 0;
											gint _tmp47_ = 0;
											gpointer* _tmp48_ = NULL;
											gpointer* _tmp49_ = NULL;
											gint _tmp49__length1 = 0;
											_tmp46_ = data;
											_tmp46__length1 = data_length1;
											_tmp48_ = gee_array_list_wrap_float (g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func, (gfloat**) _tmp46_, _tmp46__length1, &_tmp47_);
											_tmp49_ = _tmp48_;
											_tmp49__length1 = _tmp47_;
											if (result_length1) {
												*result_length1 = _tmp49__length1;
											}
											result = _tmp49_;
											data = (_vala_array_free (data, data_length1, (GDestroyNotify) g_destroy_func), NULL);
											return result;
										} else {
											GType _tmp50_ = 0UL;
											_tmp50_ = t;
											if (_tmp50_ == G_TYPE_DOUBLE) {
												gpointer* _tmp51_ = NULL;
												gint _tmp51__length1 = 0;
												gint _tmp52_ = 0;
												gpointer* _tmp53_ = NULL;
												gpointer* _tmp54_ = NULL;
												gint _tmp54__length1 = 0;
												_tmp51_ = data;
												_tmp51__length1 = data_length1;
												_tmp53_ = gee_array_list_wrap_double (g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func, (gdouble**) _tmp51_, _tmp51__length1, &_tmp52_);
												_tmp54_ = _tmp53_;
												_tmp54__length1 = _tmp52_;
												if (result_length1) {
													*result_length1 = _tmp54__length1;
												}
												result = _tmp54_;
												data = (_vala_array_free (data, data_length1, (GDestroyNotify) g_destroy_func), NULL);
												return result;
											} else {
												gpointer* _tmp55_ = NULL;
												gint _tmp55__length1 = 0;
												gpointer* _tmp56_ = NULL;
												gint _tmp56__length1 = 0;
												_tmp55_ = data;
												_tmp55__length1 = data_length1;
												data = NULL;
												data_length1 = 0;
												_tmp56_ = _tmp55_;
												_tmp56__length1 = _tmp55__length1;
												if (result_length1) {
													*result_length1 = _tmp56__length1;
												}
												result = _tmp56_;
												data = (_vala_array_free (data, data_length1, (GDestroyNotify) g_destroy_func), NULL);
												return result;
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
	data = (_vala_array_free (data, data_length1, (GDestroyNotify) g_destroy_func), NULL);
}


static gpointer* gee_array_list_wrap_bool (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gboolean* data, int data_length1, int* result_length1) {
	gpointer* result = NULL;
	gpointer* arr = NULL;
	gboolean* _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
	gpointer* _tmp1_ = NULL;
	gint arr_length1 = 0;
	gint _arr_size_ = 0;
	gpointer* _tmp13_ = NULL;
	gint _tmp13__length1 = 0;
	_tmp0_ = data;
	_tmp0__length1 = data_length1;
	_tmp1_ = g_new0 (gpointer, _tmp0__length1);
	arr = _tmp1_;
	arr_length1 = _tmp0__length1;
	_arr_size_ = arr_length1;
	{
		guint i = 0U;
		i = (guint) 0;
		{
			gboolean _tmp2_ = FALSE;
			_tmp2_ = TRUE;
			while (TRUE) {
				guint _tmp4_ = 0U;
				gboolean* _tmp5_ = NULL;
				gint _tmp5__length1 = 0;
				gpointer* _tmp6_ = NULL;
				gint _tmp6__length1 = 0;
				guint _tmp7_ = 0U;
				gboolean* _tmp8_ = NULL;
				gint _tmp8__length1 = 0;
				guint _tmp9_ = 0U;
				gboolean _tmp10_ = FALSE;
				gpointer _tmp11_ = NULL;
				gpointer _tmp12_ = NULL;
				if (!_tmp2_) {
					guint _tmp3_ = 0U;
					_tmp3_ = i;
					i = _tmp3_ + 1;
				}
				_tmp2_ = FALSE;
				_tmp4_ = i;
				_tmp5_ = data;
				_tmp5__length1 = data_length1;
				if (!(_tmp4_ < ((guint) _tmp5__length1))) {
					break;
				}
				_tmp6_ = arr;
				_tmp6__length1 = arr_length1;
				_tmp7_ = i;
				_tmp8_ = data;
				_tmp8__length1 = data_length1;
				_tmp9_ = i;
				_tmp10_ = _tmp8_[_tmp9_];
				_tmp11_ = ((_tmp10_ != NULL) && (g_dup_func != NULL)) ? g_dup_func ((gpointer) _tmp10_) : ((gpointer) _tmp10_);
				_g_destroy_func0 (_tmp6_[_tmp7_]);
				_tmp6_[_tmp7_] = _tmp11_;
				_tmp12_ = _tmp6_[_tmp7_];
			}
		}
	}
	_tmp13_ = arr;
	_tmp13__length1 = arr_length1;
	if (result_length1) {
		*result_length1 = _tmp13__length1;
	}
	result = _tmp13_;
	return result;
}


static gpointer* gee_array_list_wrap_char (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gchar* data, int data_length1, int* result_length1) {
	gpointer* result = NULL;
	gpointer* arr = NULL;
	gchar* _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
	gpointer* _tmp1_ = NULL;
	gint arr_length1 = 0;
	gint _arr_size_ = 0;
	gpointer* _tmp13_ = NULL;
	gint _tmp13__length1 = 0;
	_tmp0_ = data;
	_tmp0__length1 = data_length1;
	_tmp1_ = g_new0 (gpointer, _tmp0__length1);
	arr = _tmp1_;
	arr_length1 = _tmp0__length1;
	_arr_size_ = arr_length1;
	{
		guint i = 0U;
		i = (guint) 0;
		{
			gboolean _tmp2_ = FALSE;
			_tmp2_ = TRUE;
			while (TRUE) {
				guint _tmp4_ = 0U;
				gchar* _tmp5_ = NULL;
				gint _tmp5__length1 = 0;
				gpointer* _tmp6_ = NULL;
				gint _tmp6__length1 = 0;
				guint _tmp7_ = 0U;
				gchar* _tmp8_ = NULL;
				gint _tmp8__length1 = 0;
				guint _tmp9_ = 0U;
				gchar _tmp10_ = '\0';
				gpointer _tmp11_ = NULL;
				gpointer _tmp12_ = NULL;
				if (!_tmp2_) {
					guint _tmp3_ = 0U;
					_tmp3_ = i;
					i = _tmp3_ + 1;
				}
				_tmp2_ = FALSE;
				_tmp4_ = i;
				_tmp5_ = data;
				_tmp5__length1 = data_length1;
				if (!(_tmp4_ < ((guint) _tmp5__length1))) {
					break;
				}
				_tmp6_ = arr;
				_tmp6__length1 = arr_length1;
				_tmp7_ = i;
				_tmp8_ = data;
				_tmp8__length1 = data_length1;
				_tmp9_ = i;
				_tmp10_ = _tmp8_[_tmp9_];
				_tmp11_ = ((_tmp10_ != NULL) && (g_dup_func != NULL)) ? g_dup_func ((gpointer) _tmp10_) : ((gpointer) _tmp10_);
				_g_destroy_func0 (_tmp6_[_tmp7_]);
				_tmp6_[_tmp7_] = _tmp11_;
				_tmp12_ = _tmp6_[_tmp7_];
			}
		}
	}
	_tmp13_ = arr;
	_tmp13__length1 = arr_length1;
	if (result_length1) {
		*result_length1 = _tmp13__length1;
	}
	result = _tmp13_;
	return result;
}


static gpointer* gee_array_list_wrap_uchar (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guchar* data, int data_length1, int* result_length1) {
	gpointer* result = NULL;
	gpointer* arr = NULL;
	guchar* _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
	gpointer* _tmp1_ = NULL;
	gint arr_length1 = 0;
	gint _arr_size_ = 0;
	gpointer* _tmp13_ = NULL;
	gint _tmp13__length1 = 0;
	_tmp0_ = data;
	_tmp0__length1 = data_length1;
	_tmp1_ = g_new0 (gpointer, _tmp0__length1);
	arr = _tmp1_;
	arr_length1 = _tmp0__length1;
	_arr_size_ = arr_length1;
	{
		guint i = 0U;
		i = (guint) 0;
		{
			gboolean _tmp2_ = FALSE;
			_tmp2_ = TRUE;
			while (TRUE) {
				guint _tmp4_ = 0U;
				guchar* _tmp5_ = NULL;
				gint _tmp5__length1 = 0;
				gpointer* _tmp6_ = NULL;
				gint _tmp6__length1 = 0;
				guint _tmp7_ = 0U;
				guchar* _tmp8_ = NULL;
				gint _tmp8__length1 = 0;
				guint _tmp9_ = 0U;
				guchar _tmp10_ = '\0';
				gpointer _tmp11_ = NULL;
				gpointer _tmp12_ = NULL;
				if (!_tmp2_) {
					guint _tmp3_ = 0U;
					_tmp3_ = i;
					i = _tmp3_ + 1;
				}
				_tmp2_ = FALSE;
				_tmp4_ = i;
				_tmp5_ = data;
				_tmp5__length1 = data_length1;
				if (!(_tmp4_ < ((guint) _tmp5__length1))) {
					break;
				}
				_tmp6_ = arr;
				_tmp6__length1 = arr_length1;
				_tmp7_ = i;
				_tmp8_ = data;
				_tmp8__length1 = data_length1;
				_tmp9_ = i;
				_tmp10_ = _tmp8_[_tmp9_];
				_tmp11_ = ((_tmp10_ != NULL) && (g_dup_func != NULL)) ? g_dup_func ((gpointer) _tmp10_) : ((gpointer) _tmp10_);
				_g_destroy_func0 (_tmp6_[_tmp7_]);
				_tmp6_[_tmp7_] = _tmp11_;
				_tmp12_ = _tmp6_[_tmp7_];
			}
		}
	}
	_tmp13_ = arr;
	_tmp13__length1 = arr_length1;
	if (result_length1) {
		*result_length1 = _tmp13__length1;
	}
	result = _tmp13_;
	return result;
}


static gpointer* gee_array_list_wrap_int (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gint* data, int data_length1, int* result_length1) {
	gpointer* result = NULL;
	gpointer* arr = NULL;
	gint* _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
	gpointer* _tmp1_ = NULL;
	gint arr_length1 = 0;
	gint _arr_size_ = 0;
	gpointer* _tmp13_ = NULL;
	gint _tmp13__length1 = 0;
	_tmp0_ = data;
	_tmp0__length1 = data_length1;
	_tmp1_ = g_new0 (gpointer, _tmp0__length1);
	arr = _tmp1_;
	arr_length1 = _tmp0__length1;
	_arr_size_ = arr_length1;
	{
		guint i = 0U;
		i = (guint) 0;
		{
			gboolean _tmp2_ = FALSE;
			_tmp2_ = TRUE;
			while (TRUE) {
				guint _tmp4_ = 0U;
				gint* _tmp5_ = NULL;
				gint _tmp5__length1 = 0;
				gpointer* _tmp6_ = NULL;
				gint _tmp6__length1 = 0;
				guint _tmp7_ = 0U;
				gint* _tmp8_ = NULL;
				gint _tmp8__length1 = 0;
				guint _tmp9_ = 0U;
				gint _tmp10_ = 0;
				gpointer _tmp11_ = NULL;
				gpointer _tmp12_ = NULL;
				if (!_tmp2_) {
					guint _tmp3_ = 0U;
					_tmp3_ = i;
					i = _tmp3_ + 1;
				}
				_tmp2_ = FALSE;
				_tmp4_ = i;
				_tmp5_ = data;
				_tmp5__length1 = data_length1;
				if (!(_tmp4_ < ((guint) _tmp5__length1))) {
					break;
				}
				_tmp6_ = arr;
				_tmp6__length1 = arr_length1;
				_tmp7_ = i;
				_tmp8_ = data;
				_tmp8__length1 = data_length1;
				_tmp9_ = i;
				_tmp10_ = _tmp8_[_tmp9_];
				_tmp11_ = ((_tmp10_ != NULL) && (g_dup_func != NULL)) ? g_dup_func ((gpointer) _tmp10_) : ((gpointer) _tmp10_);
				_g_destroy_func0 (_tmp6_[_tmp7_]);
				_tmp6_[_tmp7_] = _tmp11_;
				_tmp12_ = _tmp6_[_tmp7_];
			}
		}
	}
	_tmp13_ = arr;
	_tmp13__length1 = arr_length1;
	if (result_length1) {
		*result_length1 = _tmp13__length1;
	}
	result = _tmp13_;
	return result;
}


static gpointer* gee_array_list_wrap_uint (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guint* data, int data_length1, int* result_length1) {
	gpointer* result = NULL;
	gpointer* arr = NULL;
	guint* _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
	gpointer* _tmp1_ = NULL;
	gint arr_length1 = 0;
	gint _arr_size_ = 0;
	gpointer* _tmp13_ = NULL;
	gint _tmp13__length1 = 0;
	_tmp0_ = data;
	_tmp0__length1 = data_length1;
	_tmp1_ = g_new0 (gpointer, _tmp0__length1);
	arr = _tmp1_;
	arr_length1 = _tmp0__length1;
	_arr_size_ = arr_length1;
	{
		guint i = 0U;
		i = (guint) 0;
		{
			gboolean _tmp2_ = FALSE;
			_tmp2_ = TRUE;
			while (TRUE) {
				guint _tmp4_ = 0U;
				guint* _tmp5_ = NULL;
				gint _tmp5__length1 = 0;
				gpointer* _tmp6_ = NULL;
				gint _tmp6__length1 = 0;
				guint _tmp7_ = 0U;
				guint* _tmp8_ = NULL;
				gint _tmp8__length1 = 0;
				guint _tmp9_ = 0U;
				guint _tmp10_ = 0U;
				gpointer _tmp11_ = NULL;
				gpointer _tmp12_ = NULL;
				if (!_tmp2_) {
					guint _tmp3_ = 0U;
					_tmp3_ = i;
					i = _tmp3_ + 1;
				}
				_tmp2_ = FALSE;
				_tmp4_ = i;
				_tmp5_ = data;
				_tmp5__length1 = data_length1;
				if (!(_tmp4_ < ((guint) _tmp5__length1))) {
					break;
				}
				_tmp6_ = arr;
				_tmp6__length1 = arr_length1;
				_tmp7_ = i;
				_tmp8_ = data;
				_tmp8__length1 = data_length1;
				_tmp9_ = i;
				_tmp10_ = _tmp8_[_tmp9_];
				_tmp11_ = ((_tmp10_ != NULL) && (g_dup_func != NULL)) ? g_dup_func ((gpointer) _tmp10_) : ((gpointer) _tmp10_);
				_g_destroy_func0 (_tmp6_[_tmp7_]);
				_tmp6_[_tmp7_] = _tmp11_;
				_tmp12_ = _tmp6_[_tmp7_];
			}
		}
	}
	_tmp13_ = arr;
	_tmp13__length1 = arr_length1;
	if (result_length1) {
		*result_length1 = _tmp13__length1;
	}
	result = _tmp13_;
	return result;
}


static gpointer* gee_array_list_wrap_int64 (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gint64* data, int data_length1, int* result_length1) {
	gpointer* result = NULL;
	gpointer* arr = NULL;
	gint64* _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
	gpointer* _tmp1_ = NULL;
	gint arr_length1 = 0;
	gint _arr_size_ = 0;
	gpointer* _tmp13_ = NULL;
	gint _tmp13__length1 = 0;
	_tmp0_ = data;
	_tmp0__length1 = data_length1;
	_tmp1_ = g_new0 (gpointer, _tmp0__length1);
	arr = _tmp1_;
	arr_length1 = _tmp0__length1;
	_arr_size_ = arr_length1;
	{
		guint i = 0U;
		i = (guint) 0;
		{
			gboolean _tmp2_ = FALSE;
			_tmp2_ = TRUE;
			while (TRUE) {
				guint _tmp4_ = 0U;
				gint64* _tmp5_ = NULL;
				gint _tmp5__length1 = 0;
				gpointer* _tmp6_ = NULL;
				gint _tmp6__length1 = 0;
				guint _tmp7_ = 0U;
				gint64* _tmp8_ = NULL;
				gint _tmp8__length1 = 0;
				guint _tmp9_ = 0U;
				gint64 _tmp10_ = 0LL;
				gpointer _tmp11_ = NULL;
				gpointer _tmp12_ = NULL;
				if (!_tmp2_) {
					guint _tmp3_ = 0U;
					_tmp3_ = i;
					i = _tmp3_ + 1;
				}
				_tmp2_ = FALSE;
				_tmp4_ = i;
				_tmp5_ = data;
				_tmp5__length1 = data_length1;
				if (!(_tmp4_ < ((guint) _tmp5__length1))) {
					break;
				}
				_tmp6_ = arr;
				_tmp6__length1 = arr_length1;
				_tmp7_ = i;
				_tmp8_ = data;
				_tmp8__length1 = data_length1;
				_tmp9_ = i;
				_tmp10_ = _tmp8_[_tmp9_];
				_tmp11_ = ((_tmp10_ != NULL) && (g_dup_func != NULL)) ? g_dup_func ((gpointer) _tmp10_) : ((gpointer) _tmp10_);
				_g_destroy_func0 (_tmp6_[_tmp7_]);
				_tmp6_[_tmp7_] = _tmp11_;
				_tmp12_ = _tmp6_[_tmp7_];
			}
		}
	}
	_tmp13_ = arr;
	_tmp13__length1 = arr_length1;
	if (result_length1) {
		*result_length1 = _tmp13__length1;
	}
	result = _tmp13_;
	return result;
}


static gpointer* gee_array_list_wrap_uint64 (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guint64* data, int data_length1, int* result_length1) {
	gpointer* result = NULL;
	gpointer* arr = NULL;
	guint64* _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
	gpointer* _tmp1_ = NULL;
	gint arr_length1 = 0;
	gint _arr_size_ = 0;
	gpointer* _tmp13_ = NULL;
	gint _tmp13__length1 = 0;
	_tmp0_ = data;
	_tmp0__length1 = data_length1;
	_tmp1_ = g_new0 (gpointer, _tmp0__length1);
	arr = _tmp1_;
	arr_length1 = _tmp0__length1;
	_arr_size_ = arr_length1;
	{
		guint i = 0U;
		i = (guint) 0;
		{
			gboolean _tmp2_ = FALSE;
			_tmp2_ = TRUE;
			while (TRUE) {
				guint _tmp4_ = 0U;
				guint64* _tmp5_ = NULL;
				gint _tmp5__length1 = 0;
				gpointer* _tmp6_ = NULL;
				gint _tmp6__length1 = 0;
				guint _tmp7_ = 0U;
				guint64* _tmp8_ = NULL;
				gint _tmp8__length1 = 0;
				guint _tmp9_ = 0U;
				guint64 _tmp10_ = 0ULL;
				gpointer _tmp11_ = NULL;
				gpointer _tmp12_ = NULL;
				if (!_tmp2_) {
					guint _tmp3_ = 0U;
					_tmp3_ = i;
					i = _tmp3_ + 1;
				}
				_tmp2_ = FALSE;
				_tmp4_ = i;
				_tmp5_ = data;
				_tmp5__length1 = data_length1;
				if (!(_tmp4_ < ((guint) _tmp5__length1))) {
					break;
				}
				_tmp6_ = arr;
				_tmp6__length1 = arr_length1;
				_tmp7_ = i;
				_tmp8_ = data;
				_tmp8__length1 = data_length1;
				_tmp9_ = i;
				_tmp10_ = _tmp8_[_tmp9_];
				_tmp11_ = ((_tmp10_ != NULL) && (g_dup_func != NULL)) ? g_dup_func ((gpointer) _tmp10_) : ((gpointer) _tmp10_);
				_g_destroy_func0 (_tmp6_[_tmp7_]);
				_tmp6_[_tmp7_] = _tmp11_;
				_tmp12_ = _tmp6_[_tmp7_];
			}
		}
	}
	_tmp13_ = arr;
	_tmp13__length1 = arr_length1;
	if (result_length1) {
		*result_length1 = _tmp13__length1;
	}
	result = _tmp13_;
	return result;
}


static gpointer* gee_array_list_wrap_long (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, glong* data, int data_length1, int* result_length1) {
	gpointer* result = NULL;
	gpointer* arr = NULL;
	glong* _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
	gpointer* _tmp1_ = NULL;
	gint arr_length1 = 0;
	gint _arr_size_ = 0;
	gpointer* _tmp13_ = NULL;
	gint _tmp13__length1 = 0;
	_tmp0_ = data;
	_tmp0__length1 = data_length1;
	_tmp1_ = g_new0 (gpointer, _tmp0__length1);
	arr = _tmp1_;
	arr_length1 = _tmp0__length1;
	_arr_size_ = arr_length1;
	{
		guint i = 0U;
		i = (guint) 0;
		{
			gboolean _tmp2_ = FALSE;
			_tmp2_ = TRUE;
			while (TRUE) {
				guint _tmp4_ = 0U;
				glong* _tmp5_ = NULL;
				gint _tmp5__length1 = 0;
				gpointer* _tmp6_ = NULL;
				gint _tmp6__length1 = 0;
				guint _tmp7_ = 0U;
				glong* _tmp8_ = NULL;
				gint _tmp8__length1 = 0;
				guint _tmp9_ = 0U;
				glong _tmp10_ = 0L;
				gpointer _tmp11_ = NULL;
				gpointer _tmp12_ = NULL;
				if (!_tmp2_) {
					guint _tmp3_ = 0U;
					_tmp3_ = i;
					i = _tmp3_ + 1;
				}
				_tmp2_ = FALSE;
				_tmp4_ = i;
				_tmp5_ = data;
				_tmp5__length1 = data_length1;
				if (!(_tmp4_ < ((guint) _tmp5__length1))) {
					break;
				}
				_tmp6_ = arr;
				_tmp6__length1 = arr_length1;
				_tmp7_ = i;
				_tmp8_ = data;
				_tmp8__length1 = data_length1;
				_tmp9_ = i;
				_tmp10_ = _tmp8_[_tmp9_];
				_tmp11_ = ((_tmp10_ != NULL) && (g_dup_func != NULL)) ? g_dup_func ((gpointer) _tmp10_) : ((gpointer) _tmp10_);
				_g_destroy_func0 (_tmp6_[_tmp7_]);
				_tmp6_[_tmp7_] = _tmp11_;
				_tmp12_ = _tmp6_[_tmp7_];
			}
		}
	}
	_tmp13_ = arr;
	_tmp13__length1 = arr_length1;
	if (result_length1) {
		*result_length1 = _tmp13__length1;
	}
	result = _tmp13_;
	return result;
}


static gpointer* gee_array_list_wrap_ulong (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gulong* data, int data_length1, int* result_length1) {
	gpointer* result = NULL;
	gpointer* arr = NULL;
	gulong* _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
	gpointer* _tmp1_ = NULL;
	gint arr_length1 = 0;
	gint _arr_size_ = 0;
	gpointer* _tmp13_ = NULL;
	gint _tmp13__length1 = 0;
	_tmp0_ = data;
	_tmp0__length1 = data_length1;
	_tmp1_ = g_new0 (gpointer, _tmp0__length1);
	arr = _tmp1_;
	arr_length1 = _tmp0__length1;
	_arr_size_ = arr_length1;
	{
		guint i = 0U;
		i = (guint) 0;
		{
			gboolean _tmp2_ = FALSE;
			_tmp2_ = TRUE;
			while (TRUE) {
				guint _tmp4_ = 0U;
				gulong* _tmp5_ = NULL;
				gint _tmp5__length1 = 0;
				gpointer* _tmp6_ = NULL;
				gint _tmp6__length1 = 0;
				guint _tmp7_ = 0U;
				gulong* _tmp8_ = NULL;
				gint _tmp8__length1 = 0;
				guint _tmp9_ = 0U;
				gulong _tmp10_ = 0UL;
				gpointer _tmp11_ = NULL;
				gpointer _tmp12_ = NULL;
				if (!_tmp2_) {
					guint _tmp3_ = 0U;
					_tmp3_ = i;
					i = _tmp3_ + 1;
				}
				_tmp2_ = FALSE;
				_tmp4_ = i;
				_tmp5_ = data;
				_tmp5__length1 = data_length1;
				if (!(_tmp4_ < ((guint) _tmp5__length1))) {
					break;
				}
				_tmp6_ = arr;
				_tmp6__length1 = arr_length1;
				_tmp7_ = i;
				_tmp8_ = data;
				_tmp8__length1 = data_length1;
				_tmp9_ = i;
				_tmp10_ = _tmp8_[_tmp9_];
				_tmp11_ = ((_tmp10_ != NULL) && (g_dup_func != NULL)) ? g_dup_func ((gpointer) _tmp10_) : ((gpointer) _tmp10_);
				_g_destroy_func0 (_tmp6_[_tmp7_]);
				_tmp6_[_tmp7_] = _tmp11_;
				_tmp12_ = _tmp6_[_tmp7_];
			}
		}
	}
	_tmp13_ = arr;
	_tmp13__length1 = arr_length1;
	if (result_length1) {
		*result_length1 = _tmp13__length1;
	}
	result = _tmp13_;
	return result;
}


static gpointer* gee_array_list_wrap_float (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gfloat** data, int data_length1, int* result_length1) {
	gpointer* result = NULL;
	gpointer* arr = NULL;
	gfloat** _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
	gpointer* _tmp1_ = NULL;
	gint arr_length1 = 0;
	gint _arr_size_ = 0;
	gpointer* _tmp13_ = NULL;
	gint _tmp13__length1 = 0;
	_tmp0_ = data;
	_tmp0__length1 = data_length1;
	_tmp1_ = g_new0 (gpointer, _tmp0__length1);
	arr = _tmp1_;
	arr_length1 = _tmp0__length1;
	_arr_size_ = arr_length1;
	{
		guint i = 0U;
		i = (guint) 0;
		{
			gboolean _tmp2_ = FALSE;
			_tmp2_ = TRUE;
			while (TRUE) {
				guint _tmp4_ = 0U;
				gfloat** _tmp5_ = NULL;
				gint _tmp5__length1 = 0;
				gpointer* _tmp6_ = NULL;
				gint _tmp6__length1 = 0;
				guint _tmp7_ = 0U;
				gfloat** _tmp8_ = NULL;
				gint _tmp8__length1 = 0;
				guint _tmp9_ = 0U;
				gfloat* _tmp10_ = NULL;
				gpointer _tmp11_ = NULL;
				gpointer _tmp12_ = NULL;
				if (!_tmp2_) {
					guint _tmp3_ = 0U;
					_tmp3_ = i;
					i = _tmp3_ + 1;
				}
				_tmp2_ = FALSE;
				_tmp4_ = i;
				_tmp5_ = data;
				_tmp5__length1 = data_length1;
				if (!(_tmp4_ < ((guint) _tmp5__length1))) {
					break;
				}
				_tmp6_ = arr;
				_tmp6__length1 = arr_length1;
				_tmp7_ = i;
				_tmp8_ = data;
				_tmp8__length1 = data_length1;
				_tmp9_ = i;
				_tmp10_ = _tmp8_[_tmp9_];
				_tmp11_ = ((_tmp10_ != NULL) && (g_dup_func != NULL)) ? g_dup_func ((gpointer) _tmp10_) : ((gpointer) _tmp10_);
				_g_destroy_func0 (_tmp6_[_tmp7_]);
				_tmp6_[_tmp7_] = _tmp11_;
				_tmp12_ = _tmp6_[_tmp7_];
			}
		}
	}
	_tmp13_ = arr;
	_tmp13__length1 = arr_length1;
	if (result_length1) {
		*result_length1 = _tmp13__length1;
	}
	result = _tmp13_;
	return result;
}


static gpointer* gee_array_list_wrap_double (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gdouble** data, int data_length1, int* result_length1) {
	gpointer* result = NULL;
	gpointer* arr = NULL;
	gdouble** _tmp0_ = NULL;
	gint _tmp0__length1 = 0;
	gpointer* _tmp1_ = NULL;
	gint arr_length1 = 0;
	gint _arr_size_ = 0;
	gpointer* _tmp13_ = NULL;
	gint _tmp13__length1 = 0;
	_tmp0_ = data;
	_tmp0__length1 = data_length1;
	_tmp1_ = g_new0 (gpointer, _tmp0__length1);
	arr = _tmp1_;
	arr_length1 = _tmp0__length1;
	_arr_size_ = arr_length1;
	{
		guint i = 0U;
		i = (guint) 0;
		{
			gboolean _tmp2_ = FALSE;
			_tmp2_ = TRUE;
			while (TRUE) {
				guint _tmp4_ = 0U;
				gdouble** _tmp5_ = NULL;
				gint _tmp5__length1 = 0;
				gpointer* _tmp6_ = NULL;
				gint _tmp6__length1 = 0;
				guint _tmp7_ = 0U;
				gdouble** _tmp8_ = NULL;
				gint _tmp8__length1 = 0;
				guint _tmp9_ = 0U;
				gdouble* _tmp10_ = NULL;
				gpointer _tmp11_ = NULL;
				gpointer _tmp12_ = NULL;
				if (!_tmp2_) {
					guint _tmp3_ = 0U;
					_tmp3_ = i;
					i = _tmp3_ + 1;
				}
				_tmp2_ = FALSE;
				_tmp4_ = i;
				_tmp5_ = data;
				_tmp5__length1 = data_length1;
				if (!(_tmp4_ < ((guint) _tmp5__length1))) {
					break;
				}
				_tmp6_ = arr;
				_tmp6__length1 = arr_length1;
				_tmp7_ = i;
				_tmp8_ = data;
				_tmp8__length1 = data_length1;
				_tmp9_ = i;
				_tmp10_ = _tmp8_[_tmp9_];
				_tmp11_ = ((_tmp10_ != NULL) && (g_dup_func != NULL)) ? g_dup_func ((gpointer) _tmp10_) : ((gpointer) _tmp10_);
				_g_destroy_func0 (_tmp6_[_tmp7_]);
				_tmp6_[_tmp7_] = _tmp11_;
				_tmp12_ = _tmp6_[_tmp7_];
			}
		}
	}
	_tmp13_ = arr;
	_tmp13__length1 = arr_length1;
	if (result_length1) {
		*result_length1 = _tmp13__length1;
	}
	result = _tmp13_;
	return result;
}


static gint gee_array_list_real_get_size (GeeAbstractCollection* base) {
	gint result;
	GeeArrayList* self;
	gint _tmp0_ = 0;
	self = (GeeArrayList*) base;
	_tmp0_ = self->_size;
	result = _tmp0_;
	return result;
}


static gboolean gee_array_list_real_get_read_only (GeeAbstractCollection* base) {
	gboolean result;
	GeeArrayList* self;
	self = (GeeArrayList*) base;
	result = FALSE;
	return result;
}


GeeEqualDataFunc gee_array_list_get_equal_func (GeeArrayList* self, gpointer* result_target) {
	GeeEqualDataFunc result;
	GeeFunctionsEqualDataFuncClosure* _tmp0_ = NULL;
	GeeEqualDataFunc _tmp1_ = NULL;
	void* _tmp1__target = NULL;
	GeeEqualDataFunc _tmp2_ = NULL;
	void* _tmp2__target = NULL;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_equal_func;
	_tmp1_ = _tmp0_->func;
	_tmp1__target = _tmp0_->func_target;
	_tmp2_ = _tmp1_;
	_tmp2__target = _tmp1__target;
	*result_target = _tmp2__target;
	result = _tmp2_;
	return result;
}


static void gee_array_list_set_equal_func (GeeArrayList* self, GeeEqualDataFunc value, gpointer value_target) {
	g_return_if_fail (self != NULL);
}


static gpointer _g_object_ref0 (gpointer self) {
	return self ? g_object_ref (self) : NULL;
}


static GeeArrayListIterator* gee_array_list_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeArrayList* list) {
	GeeArrayListIterator * self = NULL;
	GeeArrayList* _tmp0_ = NULL;
	GeeArrayList* _tmp1_ = NULL;
	GeeArrayList* _tmp2_ = NULL;
	gint _tmp3_ = 0;
	g_return_val_if_fail (list != NULL, NULL);
	self = (GeeArrayListIterator*) g_object_new (object_type, NULL);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	_tmp0_ = list;
	_tmp1_ = _g_object_ref0 (_tmp0_);
	_g_object_unref0 (self->_list);
	self->_list = _tmp1_;
	_tmp2_ = self->_list;
	_tmp3_ = _tmp2_->priv->_stamp;
	self->_stamp = _tmp3_;
	return self;
}


static GeeArrayListIterator* gee_array_list_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeArrayList* list) {
	return gee_array_list_iterator_construct (GEE_ARRAY_LIST_TYPE_ITERATOR, g_type, g_dup_func, g_destroy_func, list);
}


static GeeArrayListIterator* gee_array_list_iterator_construct_from_iterator (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeArrayListIterator* iter) {
	GeeArrayListIterator * self = NULL;
	GeeArrayListIterator* _tmp0_ = NULL;
	GeeArrayList* _tmp1_ = NULL;
	GeeArrayList* _tmp2_ = NULL;
	GeeArrayListIterator* _tmp3_ = NULL;
	gint _tmp4_ = 0;
	GeeArrayListIterator* _tmp5_ = NULL;
	gboolean _tmp6_ = FALSE;
	GeeArrayListIterator* _tmp7_ = NULL;
	gint _tmp8_ = 0;
	g_return_val_if_fail (iter != NULL, NULL);
	self = (GeeArrayListIterator*) g_object_new (object_type, NULL);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	_tmp0_ = iter;
	_tmp1_ = _tmp0_->_list;
	_tmp2_ = _g_object_ref0 (_tmp1_);
	_g_object_unref0 (self->_list);
	self->_list = _tmp2_;
	_tmp3_ = iter;
	_tmp4_ = _tmp3_->_index;
	self->_index = _tmp4_;
	_tmp5_ = iter;
	_tmp6_ = _tmp5_->_removed;
	self->_removed = _tmp6_;
	_tmp7_ = iter;
	_tmp8_ = _tmp7_->_stamp;
	self->_stamp = _tmp8_;
	return self;
}


static GeeArrayListIterator* gee_array_list_iterator_new_from_iterator (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeArrayListIterator* iter) {
	return gee_array_list_iterator_construct_from_iterator (GEE_ARRAY_LIST_TYPE_ITERATOR, g_type, g_dup_func, g_destroy_func, iter);
}


static gboolean gee_array_list_iterator_real_next (GeeIterator* base) {
	GeeArrayListIterator * self;
	gboolean result = FALSE;
	gint _tmp0_ = 0;
	GeeArrayList* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	gint _tmp3_ = 0;
	GeeArrayList* _tmp4_ = NULL;
	gint _tmp5_ = 0;
	self = (GeeArrayListIterator*) base;
	_tmp0_ = self->_stamp;
	_tmp1_ = self->_list;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _list._stamp");
	_tmp3_ = self->_index;
	_tmp4_ = self->_list;
	_tmp5_ = _tmp4_->_size;
	if ((_tmp3_ + 1) < _tmp5_) {
		gint _tmp6_ = 0;
		_tmp6_ = self->_index;
		self->_index = _tmp6_ + 1;
		self->_removed = FALSE;
		result = TRUE;
		return result;
	}
	result = FALSE;
	return result;
}


static gboolean gee_array_list_iterator_real_has_next (GeeIterator* base) {
	GeeArrayListIterator * self;
	gboolean result = FALSE;
	gint _tmp0_ = 0;
	GeeArrayList* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	gint _tmp3_ = 0;
	GeeArrayList* _tmp4_ = NULL;
	gint _tmp5_ = 0;
	self = (GeeArrayListIterator*) base;
	_tmp0_ = self->_stamp;
	_tmp1_ = self->_list;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _list._stamp");
	_tmp3_ = self->_index;
	_tmp4_ = self->_list;
	_tmp5_ = _tmp4_->_size;
	result = (_tmp3_ + 1) < _tmp5_;
	return result;
}


static gboolean gee_array_list_iterator_real_first (GeeBidirIterator* base) {
	GeeArrayListIterator * self;
	gboolean result = FALSE;
	gint _tmp0_ = 0;
	GeeArrayList* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	GeeArrayList* _tmp3_ = NULL;
	gint _tmp4_ = 0;
	gint _tmp5_ = 0;
	self = (GeeArrayListIterator*) base;
	_tmp0_ = self->_stamp;
	_tmp1_ = self->_list;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _list._stamp");
	_tmp3_ = self->_list;
	_tmp4_ = gee_abstract_collection_get_size ((GeeCollection*) _tmp3_);
	_tmp5_ = _tmp4_;
	if (_tmp5_ == 0) {
		result = FALSE;
		return result;
	}
	self->_index = 0;
	self->_removed = FALSE;
	result = TRUE;
	return result;
}


static gpointer gee_array_list_iterator_real_get (GeeIterator* base) {
	GeeArrayListIterator * self;
	gpointer result = NULL;
	gint _tmp0_ = 0;
	GeeArrayList* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	gboolean _tmp3_ = FALSE;
	gint _tmp4_ = 0;
	gint _tmp5_ = 0;
	GeeArrayList* _tmp6_ = NULL;
	gint _tmp7_ = 0;
	GeeArrayList* _tmp8_ = NULL;
	gpointer* _tmp9_ = NULL;
	gint _tmp9__length1 = 0;
	gint _tmp10_ = 0;
	gconstpointer _tmp11_ = NULL;
	gpointer _tmp12_ = NULL;
	self = (GeeArrayListIterator*) base;
	_tmp0_ = self->_stamp;
	_tmp1_ = self->_list;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _list._stamp");
	_tmp3_ = self->_removed;
	_vala_assert (!_tmp3_, "! _removed");
	_tmp4_ = self->_index;
	_vala_assert (_tmp4_ >= 0, "_index >= 0");
	_tmp5_ = self->_index;
	_tmp6_ = self->_list;
	_tmp7_ = _tmp6_->_size;
	_vala_assert (_tmp5_ < _tmp7_, "_index < _list._size");
	_tmp8_ = self->_list;
	_tmp9_ = _tmp8_->_items;
	_tmp9__length1 = _tmp8_->_items_length1;
	_tmp10_ = self->_index;
	_tmp11_ = _tmp9_[_tmp10_];
	_tmp12_ = ((_tmp11_ != NULL) && (self->priv->g_dup_func != NULL)) ? self->priv->g_dup_func ((gpointer) _tmp11_) : ((gpointer) _tmp11_);
	result = _tmp12_;
	return result;
}


static void gee_array_list_iterator_real_remove (GeeIterator* base) {
	GeeArrayListIterator * self;
	gint _tmp0_ = 0;
	GeeArrayList* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	gboolean _tmp3_ = FALSE;
	gboolean _tmp4_ = FALSE;
	gint _tmp6_ = 0;
	GeeArrayList* _tmp7_ = NULL;
	gint _tmp8_ = 0;
	GeeArrayList* _tmp9_ = NULL;
	gint _tmp10_ = 0;
	gpointer _tmp11_ = NULL;
	gpointer _tmp12_ = NULL;
	gint _tmp13_ = 0;
	GeeArrayList* _tmp14_ = NULL;
	gint _tmp15_ = 0;
	self = (GeeArrayListIterator*) base;
	_tmp0_ = self->_stamp;
	_tmp1_ = self->_list;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _list._stamp");
	_tmp4_ = self->_removed;
	if (!_tmp4_) {
		gint _tmp5_ = 0;
		_tmp5_ = self->_index;
		_tmp3_ = _tmp5_ >= 0;
	} else {
		_tmp3_ = FALSE;
	}
	_vala_assert (_tmp3_, "! _removed && _index >= 0");
	_tmp6_ = self->_index;
	_tmp7_ = self->_list;
	_tmp8_ = _tmp7_->_size;
	_vala_assert (_tmp6_ < _tmp8_, "_index < _list._size");
	_tmp9_ = self->_list;
	_tmp10_ = self->_index;
	_tmp11_ = gee_abstract_list_remove_at ((GeeAbstractList*) _tmp9_, _tmp10_);
	_tmp12_ = _tmp11_;
	((_tmp12_ == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (_tmp12_ = (self->priv->g_destroy_func (_tmp12_), NULL));
	_tmp13_ = self->_index;
	self->_index = _tmp13_ - 1;
	self->_removed = TRUE;
	_tmp14_ = self->_list;
	_tmp15_ = _tmp14_->priv->_stamp;
	self->_stamp = _tmp15_;
}


static gboolean gee_array_list_iterator_real_previous (GeeBidirIterator* base) {
	GeeArrayListIterator * self;
	gboolean result = FALSE;
	gint _tmp0_ = 0;
	GeeArrayList* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	gboolean _tmp3_ = FALSE;
	gboolean _tmp4_ = FALSE;
	gint _tmp6_ = 0;
	self = (GeeArrayListIterator*) base;
	_tmp0_ = self->_stamp;
	_tmp1_ = self->_list;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _list._stamp");
	_tmp4_ = self->_removed;
	if (_tmp4_) {
		gint _tmp5_ = 0;
		_tmp5_ = self->_index;
		_tmp3_ = _tmp5_ >= 0;
	} else {
		_tmp3_ = FALSE;
	}
	if (_tmp3_) {
		self->_removed = FALSE;
		result = TRUE;
		return result;
	}
	_tmp6_ = self->_index;
	if (_tmp6_ > 0) {
		gint _tmp7_ = 0;
		_tmp7_ = self->_index;
		self->_index = _tmp7_ - 1;
		result = TRUE;
		return result;
	}
	result = FALSE;
	return result;
}


static gboolean gee_array_list_iterator_real_has_previous (GeeBidirIterator* base) {
	GeeArrayListIterator * self;
	gboolean result = FALSE;
	gint _tmp0_ = 0;
	GeeArrayList* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	gboolean _tmp3_ = FALSE;
	gint _tmp4_ = 0;
	self = (GeeArrayListIterator*) base;
	_tmp0_ = self->_stamp;
	_tmp1_ = self->_list;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _list._stamp");
	_tmp4_ = self->_index;
	if (_tmp4_ > 0) {
		_tmp3_ = TRUE;
	} else {
		gboolean _tmp5_ = FALSE;
		gboolean _tmp6_ = FALSE;
		_tmp6_ = self->_removed;
		if (_tmp6_) {
			gint _tmp7_ = 0;
			_tmp7_ = self->_index;
			_tmp5_ = _tmp7_ >= 0;
		} else {
			_tmp5_ = FALSE;
		}
		_tmp3_ = _tmp5_;
	}
	result = _tmp3_;
	return result;
}


static gboolean gee_array_list_iterator_real_last (GeeBidirIterator* base) {
	GeeArrayListIterator * self;
	gboolean result = FALSE;
	gint _tmp0_ = 0;
	GeeArrayList* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	GeeArrayList* _tmp3_ = NULL;
	gint _tmp4_ = 0;
	gint _tmp5_ = 0;
	GeeArrayList* _tmp6_ = NULL;
	gint _tmp7_ = 0;
	self = (GeeArrayListIterator*) base;
	_tmp0_ = self->_stamp;
	_tmp1_ = self->_list;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _list._stamp");
	_tmp3_ = self->_list;
	_tmp4_ = gee_abstract_collection_get_size ((GeeCollection*) _tmp3_);
	_tmp5_ = _tmp4_;
	if (_tmp5_ == 0) {
		result = FALSE;
		return result;
	}
	_tmp6_ = self->_list;
	_tmp7_ = _tmp6_->_size;
	self->_index = _tmp7_ - 1;
	result = TRUE;
	return result;
}


static void gee_array_list_iterator_real_set (GeeListIterator* base, gconstpointer item) {
	GeeArrayListIterator * self;
	gint _tmp0_ = 0;
	GeeArrayList* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	gboolean _tmp3_ = FALSE;
	gint _tmp4_ = 0;
	gint _tmp5_ = 0;
	GeeArrayList* _tmp6_ = NULL;
	gint _tmp7_ = 0;
	GeeArrayList* _tmp8_ = NULL;
	gpointer* _tmp9_ = NULL;
	gint _tmp9__length1 = 0;
	gint _tmp10_ = 0;
	gconstpointer _tmp11_ = NULL;
	gpointer _tmp12_ = NULL;
	gpointer _tmp13_ = NULL;
	GeeArrayList* _tmp14_ = NULL;
	GeeArrayList* _tmp15_ = NULL;
	gint _tmp16_ = 0;
	gint _tmp17_ = 0;
	self = (GeeArrayListIterator*) base;
	_tmp0_ = self->_stamp;
	_tmp1_ = self->_list;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _list._stamp");
	_tmp3_ = self->_removed;
	_vala_assert (!_tmp3_, "! _removed");
	_tmp4_ = self->_index;
	_vala_assert (_tmp4_ >= 0, "_index >= 0");
	_tmp5_ = self->_index;
	_tmp6_ = self->_list;
	_tmp7_ = _tmp6_->_size;
	_vala_assert (_tmp5_ < _tmp7_, "_index < _list._size");
	_tmp8_ = self->_list;
	_tmp9_ = _tmp8_->_items;
	_tmp9__length1 = _tmp8_->_items_length1;
	_tmp10_ = self->_index;
	_tmp11_ = item;
	_tmp12_ = ((_tmp11_ != NULL) && (self->priv->g_dup_func != NULL)) ? self->priv->g_dup_func ((gpointer) _tmp11_) : ((gpointer) _tmp11_);
	((_tmp9_[_tmp10_] == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (_tmp9_[_tmp10_] = (self->priv->g_destroy_func (_tmp9_[_tmp10_]), NULL));
	_tmp9_[_tmp10_] = _tmp12_;
	_tmp13_ = _tmp9_[_tmp10_];
	_tmp14_ = self->_list;
	_tmp15_ = self->_list;
	_tmp16_ = _tmp15_->priv->_stamp;
	_tmp15_->priv->_stamp = _tmp16_ + 1;
	_tmp17_ = _tmp15_->priv->_stamp;
	self->_stamp = _tmp17_;
}


static void gee_array_list_iterator_real_insert (GeeBidirListIterator* base, gconstpointer item) {
	GeeArrayListIterator * self;
	gint _tmp0_ = 0;
	GeeArrayList* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	gint _tmp3_ = 0;
	GeeArrayList* _tmp4_ = NULL;
	gint _tmp5_ = 0;
	gint _tmp6_ = 0;
	gboolean _tmp9_ = FALSE;
	gint _tmp16_ = 0;
	GeeArrayList* _tmp17_ = NULL;
	gint _tmp18_ = 0;
	self = (GeeArrayListIterator*) base;
	_tmp0_ = self->_stamp;
	_tmp1_ = self->_list;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _list._stamp");
	_tmp3_ = self->_index;
	_tmp4_ = self->_list;
	_tmp5_ = _tmp4_->_size;
	_vala_assert (_tmp3_ < _tmp5_, "_index < _list._size");
	_tmp6_ = self->_index;
	if (_tmp6_ == (-1)) {
		GeeArrayList* _tmp7_ = NULL;
		gconstpointer _tmp8_ = NULL;
		_tmp7_ = self->_list;
		_tmp8_ = item;
		gee_abstract_list_insert ((GeeAbstractList*) _tmp7_, 0, _tmp8_);
		self->_removed = TRUE;
	}
	_tmp9_ = self->_removed;
	if (_tmp9_) {
		GeeArrayList* _tmp10_ = NULL;
		gint _tmp11_ = 0;
		gconstpointer _tmp12_ = NULL;
		_tmp10_ = self->_list;
		_tmp11_ = self->_index;
		_tmp12_ = item;
		gee_abstract_list_insert ((GeeAbstractList*) _tmp10_, _tmp11_ + 1, _tmp12_);
	} else {
		GeeArrayList* _tmp13_ = NULL;
		gint _tmp14_ = 0;
		gconstpointer _tmp15_ = NULL;
		_tmp13_ = self->_list;
		_tmp14_ = self->_index;
		_tmp15_ = item;
		gee_abstract_list_insert ((GeeAbstractList*) _tmp13_, _tmp14_, _tmp15_);
	}
	_tmp16_ = self->_index;
	self->_index = _tmp16_ + 1;
	_tmp17_ = self->_list;
	_tmp18_ = _tmp17_->priv->_stamp;
	self->_stamp = _tmp18_;
}


static void gee_array_list_iterator_real_add (GeeListIterator* base, gconstpointer item) {
	GeeArrayListIterator * self;
	gint _tmp0_ = 0;
	GeeArrayList* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	gint _tmp3_ = 0;
	GeeArrayList* _tmp4_ = NULL;
	gint _tmp5_ = 0;
	GeeArrayList* _tmp6_ = NULL;
	gint _tmp7_ = 0;
	gconstpointer _tmp8_ = NULL;
	gint _tmp9_ = 0;
	GeeArrayList* _tmp10_ = NULL;
	gint _tmp11_ = 0;
	self = (GeeArrayListIterator*) base;
	_tmp0_ = self->_stamp;
	_tmp1_ = self->_list;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _list._stamp");
	_tmp3_ = self->_index;
	_tmp4_ = self->_list;
	_tmp5_ = _tmp4_->_size;
	_vala_assert (_tmp3_ < _tmp5_, "_index < _list._size");
	_tmp6_ = self->_list;
	_tmp7_ = self->_index;
	_tmp8_ = item;
	gee_abstract_list_insert ((GeeAbstractList*) _tmp6_, _tmp7_ + 1, _tmp8_);
	_tmp9_ = self->_index;
	self->_index = _tmp9_ + 1;
	self->_removed = FALSE;
	_tmp10_ = self->_list;
	_tmp11_ = _tmp10_->priv->_stamp;
	self->_stamp = _tmp11_;
}


static gint gee_array_list_iterator_real_index (GeeListIterator* base) {
	GeeArrayListIterator * self;
	gint result = 0;
	gint _tmp0_ = 0;
	GeeArrayList* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	gint _tmp3_ = 0;
	gint _tmp4_ = 0;
	GeeArrayList* _tmp5_ = NULL;
	gint _tmp6_ = 0;
	gint _tmp7_ = 0;
	self = (GeeArrayListIterator*) base;
	_tmp0_ = self->_stamp;
	_tmp1_ = self->_list;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _list._stamp");
	_tmp3_ = self->_index;
	_vala_assert (_tmp3_ >= 0, "_index >= 0");
	_tmp4_ = self->_index;
	_tmp5_ = self->_list;
	_tmp6_ = _tmp5_->_size;
	_vala_assert (_tmp4_ < _tmp6_, "_index < _list._size");
	_tmp7_ = self->_index;
	result = _tmp7_;
	return result;
}


static gboolean gee_array_list_iterator_real_foreach (GeeTraversable* base, GeeForallFunc f, void* f_target) {
	GeeArrayListIterator * self;
	gboolean result = FALSE;
	gint _tmp0_ = 0;
	GeeArrayList* _tmp1_ = NULL;
	gint _tmp2_ = 0;
	gboolean _tmp3_ = FALSE;
	gint _tmp4_ = 0;
	GeeArrayList* _tmp18_ = NULL;
	gint _tmp19_ = 0;
	self = (GeeArrayListIterator*) base;
	_tmp0_ = self->_stamp;
	_tmp1_ = self->_list;
	_tmp2_ = _tmp1_->priv->_stamp;
	_vala_assert (_tmp0_ == _tmp2_, "_stamp == _list._stamp");
	_tmp4_ = self->_index;
	if (_tmp4_ < 0) {
		_tmp3_ = TRUE;
	} else {
		gboolean _tmp5_ = FALSE;
		_tmp5_ = self->_removed;
		_tmp3_ = _tmp5_;
	}
	if (_tmp3_) {
		gint _tmp6_ = 0;
		_tmp6_ = self->_index;
		self->_index = _tmp6_ + 1;
	}
	while (TRUE) {
		gint _tmp7_ = 0;
		GeeArrayList* _tmp8_ = NULL;
		gint _tmp9_ = 0;
		GeeForallFunc _tmp10_ = NULL;
		void* _tmp10__target = NULL;
		GeeArrayList* _tmp11_ = NULL;
		gpointer* _tmp12_ = NULL;
		gint _tmp12__length1 = 0;
		gint _tmp13_ = 0;
		gconstpointer _tmp14_ = NULL;
		gpointer _tmp15_ = NULL;
		gboolean _tmp16_ = FALSE;
		gint _tmp17_ = 0;
		_tmp7_ = self->_index;
		_tmp8_ = self->_list;
		_tmp9_ = _tmp8_->_size;
		if (!(_tmp7_ < _tmp9_)) {
			break;
		}
		_tmp10_ = f;
		_tmp10__target = f_target;
		_tmp11_ = self->_list;
		_tmp12_ = _tmp11_->_items;
		_tmp12__length1 = _tmp11_->_items_length1;
		_tmp13_ = self->_index;
		_tmp14_ = _tmp12_[_tmp13_];
		_tmp15_ = ((_tmp14_ != NULL) && (self->priv->g_dup_func != NULL)) ? self->priv->g_dup_func ((gpointer) _tmp14_) : ((gpointer) _tmp14_);
		_tmp16_ = _tmp10_ (_tmp15_, _tmp10__target);
		if (!_tmp16_) {
			result = FALSE;
			return result;
		}
		_tmp17_ = self->_index;
		self->_index = _tmp17_ + 1;
	}
	_tmp18_ = self->_list;
	_tmp19_ = _tmp18_->_size;
	self->_index = _tmp19_ - 1;
	result = TRUE;
	return result;
}


static GeeIterator** gee_array_list_iterator_real_tee (GeeTraversable* base, guint forks, int* result_length1) {
	GeeArrayListIterator * self;
	GeeIterator** result = NULL;
	guint _tmp0_ = 0U;
	self = (GeeArrayListIterator*) base;
	_tmp0_ = forks;
	if (_tmp0_ == ((guint) 0)) {
		GeeIterator** _tmp1_ = NULL;
		GeeIterator** _tmp2_ = NULL;
		gint _tmp2__length1 = 0;
		_tmp1_ = g_new0 (GeeIterator*, 0 + 1);
		_tmp2_ = _tmp1_;
		_tmp2__length1 = 0;
		if (result_length1) {
			*result_length1 = _tmp2__length1;
		}
		result = _tmp2_;
		return result;
	} else {
		GeeIterator** _result_ = NULL;
		guint _tmp3_ = 0U;
		GeeIterator** _tmp4_ = NULL;
		gint _result__length1 = 0;
		gint __result__size_ = 0;
		GeeIterator** _tmp5_ = NULL;
		gint _tmp5__length1 = 0;
		GeeIterator* _tmp6_ = NULL;
		GeeIterator* _tmp7_ = NULL;
		GeeIterator** _tmp16_ = NULL;
		gint _tmp16__length1 = 0;
		_tmp3_ = forks;
		_tmp4_ = g_new0 (GeeIterator*, _tmp3_ + 1);
		_result_ = _tmp4_;
		_result__length1 = _tmp3_;
		__result__size_ = _result__length1;
		_tmp5_ = _result_;
		_tmp5__length1 = _result__length1;
		_tmp6_ = _g_object_ref0 ((GeeIterator*) self);
		_g_object_unref0 (_tmp5_[0]);
		_tmp5_[0] = _tmp6_;
		_tmp7_ = _tmp5_[0];
		{
			guint i = 0U;
			i = (guint) 1;
			{
				gboolean _tmp8_ = FALSE;
				_tmp8_ = TRUE;
				while (TRUE) {
					guint _tmp10_ = 0U;
					guint _tmp11_ = 0U;
					GeeIterator** _tmp12_ = NULL;
					gint _tmp12__length1 = 0;
					guint _tmp13_ = 0U;
					GeeArrayListIterator* _tmp14_ = NULL;
					GeeIterator* _tmp15_ = NULL;
					if (!_tmp8_) {
						guint _tmp9_ = 0U;
						_tmp9_ = i;
						i = _tmp9_ + 1;
					}
					_tmp8_ = FALSE;
					_tmp10_ = i;
					_tmp11_ = forks;
					if (!(_tmp10_ < _tmp11_)) {
						break;
					}
					_tmp12_ = _result_;
					_tmp12__length1 = _result__length1;
					_tmp13_ = i;
					_tmp14_ = gee_array_list_iterator_new_from_iterator (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, self);
					_g_object_unref0 (_tmp12_[_tmp13_]);
					_tmp12_[_tmp13_] = (GeeIterator*) _tmp14_;
					_tmp15_ = _tmp12_[_tmp13_];
				}
			}
		}
		_tmp16_ = _result_;
		_tmp16__length1 = _result__length1;
		if (result_length1) {
			*result_length1 = _tmp16__length1;
		}
		result = _tmp16_;
		return result;
	}
}


static gboolean gee_array_list_iterator_real_get_read_only (GeeIterator* base) {
	gboolean result;
	GeeArrayListIterator* self;
	self = (GeeArrayListIterator*) base;
	result = FALSE;
	return result;
}


static gboolean gee_array_list_iterator_real_get_valid (GeeIterator* base) {
	gboolean result;
	GeeArrayListIterator* self;
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	gint _tmp2_ = 0;
	self = (GeeArrayListIterator*) base;
	_tmp2_ = self->_index;
	if (_tmp2_ >= 0) {
		gint _tmp3_ = 0;
		GeeArrayList* _tmp4_ = NULL;
		gint _tmp5_ = 0;
		_tmp3_ = self->_index;
		_tmp4_ = self->_list;
		_tmp5_ = _tmp4_->_size;
		_tmp1_ = _tmp3_ < _tmp5_;
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		gboolean _tmp6_ = FALSE;
		_tmp6_ = self->_removed;
		_tmp0_ = !_tmp6_;
	} else {
		_tmp0_ = FALSE;
	}
	result = _tmp0_;
	return result;
}


static void gee_array_list_iterator_class_init (GeeArrayListIteratorClass * klass) {
	gee_array_list_iterator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeArrayListIteratorPrivate));
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_array_list_iterator_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_array_list_iterator_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_array_list_iterator_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ARRAY_LIST_ITERATOR_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ARRAY_LIST_ITERATOR_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ARRAY_LIST_ITERATOR_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ARRAY_LIST_ITERATOR_READ_ONLY, g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ARRAY_LIST_ITERATOR_VALID, g_param_spec_boolean ("valid", "valid", "valid", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static GType gee_array_list_iterator_gee_traversable_get_g_type (GeeArrayListIterator* self) {
	return self->priv->g_type;
}


static GBoxedCopyFunc gee_array_list_iterator_gee_traversable_get_g_dup_func (GeeArrayListIterator* self) {
	return self->priv->g_dup_func;
}


static GDestroyNotify gee_array_list_iterator_gee_traversable_get_g_destroy_func (GeeArrayListIterator* self) {
	return self->priv->g_destroy_func;
}


static void gee_array_list_iterator_gee_traversable_interface_init (GeeTraversableIface * iface) {
	gee_array_list_iterator_gee_traversable_parent_iface = g_type_interface_peek_parent (iface);
	iface->foreach = (gboolean (*)(GeeTraversable*, GeeForallFunc, void*)) gee_array_list_iterator_real_foreach;
	iface->tee = (GeeIterator** (*)(GeeTraversable*, guint, int*)) gee_array_list_iterator_real_tee;
	iface->get_g_type = (GType(*)(GeeTraversable*)) gee_array_list_iterator_gee_traversable_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc(*)(GeeTraversable*)) gee_array_list_iterator_gee_traversable_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify(*)(GeeTraversable*)) gee_array_list_iterator_gee_traversable_get_g_destroy_func;
}


static void gee_array_list_iterator_gee_iterator_interface_init (GeeIteratorIface * iface) {
	gee_array_list_iterator_gee_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->next = (gboolean (*)(GeeIterator*)) gee_array_list_iterator_real_next;
	iface->has_next = (gboolean (*)(GeeIterator*)) gee_array_list_iterator_real_has_next;
	iface->get = (gpointer (*)(GeeIterator*)) gee_array_list_iterator_real_get;
	iface->remove = (void (*)(GeeIterator*)) gee_array_list_iterator_real_remove;
	iface->get_read_only = gee_array_list_iterator_real_get_read_only;
	iface->get_valid = gee_array_list_iterator_real_get_valid;
}


static GType gee_array_list_iterator_gee_bidir_iterator_get_g_type (GeeArrayListIterator* self) {
	return self->priv->g_type;
}


static GBoxedCopyFunc gee_array_list_iterator_gee_bidir_iterator_get_g_dup_func (GeeArrayListIterator* self) {
	return self->priv->g_dup_func;
}


static GDestroyNotify gee_array_list_iterator_gee_bidir_iterator_get_g_destroy_func (GeeArrayListIterator* self) {
	return self->priv->g_destroy_func;
}


static void gee_array_list_iterator_gee_bidir_iterator_interface_init (GeeBidirIteratorIface * iface) {
	gee_array_list_iterator_gee_bidir_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->first = (gboolean (*)(GeeBidirIterator*)) gee_array_list_iterator_real_first;
	iface->previous = (gboolean (*)(GeeBidirIterator*)) gee_array_list_iterator_real_previous;
	iface->has_previous = (gboolean (*)(GeeBidirIterator*)) gee_array_list_iterator_real_has_previous;
	iface->last = (gboolean (*)(GeeBidirIterator*)) gee_array_list_iterator_real_last;
	iface->get_g_type = (GType(*)(GeeBidirIterator*)) gee_array_list_iterator_gee_bidir_iterator_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc(*)(GeeBidirIterator*)) gee_array_list_iterator_gee_bidir_iterator_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify(*)(GeeBidirIterator*)) gee_array_list_iterator_gee_bidir_iterator_get_g_destroy_func;
}


static void gee_array_list_iterator_gee_list_iterator_interface_init (GeeListIteratorIface * iface) {
	gee_array_list_iterator_gee_list_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->set = (void (*)(GeeListIterator*, gconstpointer)) gee_array_list_iterator_real_set;
	iface->add = (void (*)(GeeListIterator*, gconstpointer)) gee_array_list_iterator_real_add;
	iface->index = (gint (*)(GeeListIterator*)) gee_array_list_iterator_real_index;
}


static GType gee_array_list_iterator_gee_bidir_list_iterator_get_g_type (GeeArrayListIterator* self) {
	return self->priv->g_type;
}


static GBoxedCopyFunc gee_array_list_iterator_gee_bidir_list_iterator_get_g_dup_func (GeeArrayListIterator* self) {
	return self->priv->g_dup_func;
}


static GDestroyNotify gee_array_list_iterator_gee_bidir_list_iterator_get_g_destroy_func (GeeArrayListIterator* self) {
	return self->priv->g_destroy_func;
}


static void gee_array_list_iterator_gee_bidir_list_iterator_interface_init (GeeBidirListIteratorIface * iface) {
	gee_array_list_iterator_gee_bidir_list_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->insert = (void (*)(GeeBidirListIterator*, gconstpointer)) gee_array_list_iterator_real_insert;
	iface->get_g_type = (GType(*)(GeeBidirListIterator*)) gee_array_list_iterator_gee_bidir_list_iterator_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc(*)(GeeBidirListIterator*)) gee_array_list_iterator_gee_bidir_list_iterator_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify(*)(GeeBidirListIterator*)) gee_array_list_iterator_gee_bidir_list_iterator_get_g_destroy_func;
}


static void gee_array_list_iterator_instance_init (GeeArrayListIterator * self) {
	self->priv = GEE_ARRAY_LIST_ITERATOR_GET_PRIVATE (self);
	self->_index = -1;
	self->_removed = FALSE;
	self->_stamp = 0;
}


static void gee_array_list_iterator_finalize (GObject* obj) {
	GeeArrayListIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_ARRAY_LIST_TYPE_ITERATOR, GeeArrayListIterator);
	_g_object_unref0 (self->_list);
	G_OBJECT_CLASS (gee_array_list_iterator_parent_class)->finalize (obj);
}


static GType gee_array_list_iterator_get_type (void) {
	static volatile gsize gee_array_list_iterator_type_id__volatile = 0;
	if (g_once_init_enter (&gee_array_list_iterator_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeArrayListIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_array_list_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeArrayListIterator), 0, (GInstanceInitFunc) gee_array_list_iterator_instance_init, NULL };
		static const GInterfaceInfo gee_traversable_info = { (GInterfaceInitFunc) gee_array_list_iterator_gee_traversable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo gee_iterator_info = { (GInterfaceInitFunc) gee_array_list_iterator_gee_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo gee_bidir_iterator_info = { (GInterfaceInitFunc) gee_array_list_iterator_gee_bidir_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo gee_list_iterator_info = { (GInterfaceInitFunc) gee_array_list_iterator_gee_list_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo gee_bidir_list_iterator_info = { (GInterfaceInitFunc) gee_array_list_iterator_gee_bidir_list_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType gee_array_list_iterator_type_id;
		gee_array_list_iterator_type_id = g_type_register_static (G_TYPE_OBJECT, "GeeArrayListIterator", &g_define_type_info, 0);
		g_type_add_interface_static (gee_array_list_iterator_type_id, GEE_TYPE_TRAVERSABLE, &gee_traversable_info);
		g_type_add_interface_static (gee_array_list_iterator_type_id, GEE_TYPE_ITERATOR, &gee_iterator_info);
		g_type_add_interface_static (gee_array_list_iterator_type_id, GEE_TYPE_BIDIR_ITERATOR, &gee_bidir_iterator_info);
		g_type_add_interface_static (gee_array_list_iterator_type_id, GEE_TYPE_LIST_ITERATOR, &gee_list_iterator_info);
		g_type_add_interface_static (gee_array_list_iterator_type_id, GEE_TYPE_BIDIR_LIST_ITERATOR, &gee_bidir_list_iterator_info);
		g_once_init_leave (&gee_array_list_iterator_type_id__volatile, gee_array_list_iterator_type_id);
	}
	return gee_array_list_iterator_type_id__volatile;
}


static void _vala_gee_array_list_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeArrayListIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_ARRAY_LIST_TYPE_ITERATOR, GeeArrayListIterator);
	switch (property_id) {
		case GEE_ARRAY_LIST_ITERATOR_READ_ONLY:
		g_value_set_boolean (value, gee_iterator_get_read_only ((GeeIterator*) self));
		break;
		case GEE_ARRAY_LIST_ITERATOR_VALID:
		g_value_set_boolean (value, gee_iterator_get_valid ((GeeIterator*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_gee_array_list_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeArrayListIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_ARRAY_LIST_TYPE_ITERATOR, GeeArrayListIterator);
	switch (property_id) {
		case GEE_ARRAY_LIST_ITERATOR_G_TYPE:
		self->priv->g_type = g_value_get_gtype (value);
		break;
		case GEE_ARRAY_LIST_ITERATOR_G_DUP_FUNC:
		self->priv->g_dup_func = g_value_get_pointer (value);
		break;
		case GEE_ARRAY_LIST_ITERATOR_G_DESTROY_FUNC:
		self->priv->g_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gee_array_list_class_init (GeeArrayListClass * klass) {
	gee_array_list_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeArrayListPrivate));
	((GeeAbstractCollectionClass *) klass)->foreach = gee_array_list_real_foreach;
	((GeeAbstractCollectionClass *) klass)->iterator = gee_array_list_real_iterator;
	((GeeAbstractListClass *) klass)->list_iterator = gee_array_list_real_list_iterator;
	((GeeAbstractBidirListClass *) klass)->bidir_list_iterator = gee_array_list_real_bidir_list_iterator;
	((GeeAbstractCollectionClass *) klass)->contains = gee_array_list_real_contains;
	((GeeAbstractListClass *) klass)->index_of = gee_array_list_real_index_of;
	((GeeAbstractListClass *) klass)->get = gee_array_list_real_get;
	((GeeAbstractListClass *) klass)->set = gee_array_list_real_set;
	((GeeAbstractCollectionClass *) klass)->add = gee_array_list_real_add;
	((GeeAbstractListClass *) klass)->insert = gee_array_list_real_insert;
	((GeeAbstractCollectionClass *) klass)->remove = gee_array_list_real_remove;
	((GeeAbstractListClass *) klass)->remove_at = gee_array_list_real_remove_at;
	((GeeAbstractCollectionClass *) klass)->clear = gee_array_list_real_clear;
	((GeeAbstractListClass *) klass)->slice = gee_array_list_real_slice;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->get_size = gee_array_list_real_get_size;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->get_read_only = gee_array_list_real_get_read_only;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_array_list_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_array_list_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_array_list_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ARRAY_LIST_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ARRAY_LIST_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ARRAY_LIST_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ARRAY_LIST_SIZE, g_param_spec_int ("size", "size", "size", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ARRAY_LIST_READ_ONLY, g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void gee_array_list_instance_init (GeeArrayList * self) {
	self->priv = GEE_ARRAY_LIST_GET_PRIVATE (self);
	self->priv->_stamp = 0;
}


static void gee_array_list_finalize (GObject* obj) {
	GeeArrayList * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_TYPE_ARRAY_LIST, GeeArrayList);
	self->_items = (_vala_array_free (self->_items, self->_items_length1, (GDestroyNotify) self->priv->g_destroy_func), NULL);
	_gee_functions_equal_data_func_closure_unref0 (self->priv->_equal_func);
	G_OBJECT_CLASS (gee_array_list_parent_class)->finalize (obj);
}


/**
 * Resizable array implementation of the {@link List} interface.
 *
 * The storage array grows automatically when needed.
 *
 * This implementation is pretty good for rarely modified data. Because they are
 * stored in an array this structure does not fit for highly mutable data. For an
 * alternative implementation see {@link LinkedList}.
 *
 * @see LinkedList
 */
GType gee_array_list_get_type (void) {
	static volatile gsize gee_array_list_type_id__volatile = 0;
	if (g_once_init_enter (&gee_array_list_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeArrayListClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_array_list_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeArrayList), 0, (GInstanceInitFunc) gee_array_list_instance_init, NULL };
		GType gee_array_list_type_id;
		gee_array_list_type_id = g_type_register_static (GEE_TYPE_ABSTRACT_BIDIR_LIST, "GeeArrayList", &g_define_type_info, 0);
		g_once_init_leave (&gee_array_list_type_id__volatile, gee_array_list_type_id);
	}
	return gee_array_list_type_id__volatile;
}


static void _vala_gee_array_list_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeArrayList * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_ARRAY_LIST, GeeArrayList);
	switch (property_id) {
		case GEE_ARRAY_LIST_SIZE:
		g_value_set_int (value, gee_abstract_collection_get_size ((GeeAbstractCollection*) self));
		break;
		case GEE_ARRAY_LIST_READ_ONLY:
		g_value_set_boolean (value, gee_abstract_collection_get_read_only ((GeeAbstractCollection*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_gee_array_list_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeArrayList * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_ARRAY_LIST, GeeArrayList);
	switch (property_id) {
		case GEE_ARRAY_LIST_G_TYPE:
		self->priv->g_type = g_value_get_gtype (value);
		break;
		case GEE_ARRAY_LIST_G_DUP_FUNC:
		self->priv->g_dup_func = g_value_get_pointer (value);
		break;
		case GEE_ARRAY_LIST_G_DESTROY_FUNC:
		self->priv->g_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static void _vala_array_move (gpointer array, gsize element_size, gint src, gint dest, gint length) {
	g_memmove (((char*) array) + (dest * element_size), ((char*) array) + (src * element_size), length * element_size);
	if ((src < dest) && ((src + length) > dest)) {
		memset (((char*) array) + (src * element_size), 0, (dest - src) * element_size);
	} else if ((src > dest) && (src < (dest + length))) {
		memset (((char*) array) + ((dest + length) * element_size), 0, (src - dest) * element_size);
	} else if (src != dest) {
		memset (((char*) array) + (src * element_size), 0, length * element_size);
	}
}



